<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class MensagensController extends Crud {

    protected $dataSourceName = 'des_mensagenspadrao';
    protected $dataSourceName2 = 'des_mensagenspadrao_busca';

    protected $parameterBagColumns = array('NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getMensagens (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName2);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarMensagemPadrao (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    if(array_key_exists('NRMENSAGENSPADRAO', $row)){
                        if($row['NRMENSAGENSPADRAO'] === '' ){
                            $row['NRMENSAGENSPADRAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_MENSAGENSPADRAO', 1, 12, true);
                        }
                    } else {
                        $row['NRMENSAGENSPADRAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_MENSAGENSPADRAO', 1, 12, true);
                    }
                }

                /*if(array_key_exists('DSMENSAGEM', $row)){
                    $row['DSMENSAGEM'] = nl2br($row['DSMENSAGEM']); 
                }*/

                if(!array_key_exists('DTINCLUSAO', $row)){
                  $row['DTINCLUSAO'] = date('d/m/Y');
                } else {
                    $explode = explode(" ", $row['DTINCLUSAO']);
                    if(isset($explode[1])){
                        $row['DTINCLUSAO'] = $explode[0];
                    }
                }

                if(!array_key_exists('DTULTATU', $row)){
                  $row['DTULTATU'] = date('d/m/Y');
                } else {
                    $explode = explode(" ", $row['DTULTATU']);
                    if(isset($explode[1])){
                        $row['DTULTATU'] = $explode[0];
                    }
                }

                if(!array_key_exists('NRORGINCLUSAO', $row)){
                    $row['NRORGINCLUSAO'] = $row['NRORG'];
                }

                if(!array_key_exists('NRORGULTATU', $row)){
                    $row['NRORGULTATU'] = $row['NRORG'];
                }

                if(!array_key_exists('CDOPERINCLUSAO', $row)){
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                }

                if(!array_key_exists('CDOPERULTATU', $row)){
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                }
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}