<?php

namespace HCM\FPA\Calculos\Geral;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpaeventohEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;

class MediaEventosIntervalo{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mes_ocorrencia;
    private $_dtPagamento;
    private $_evento;
    private $_calculo_folha;
    private $_itens_calculo_fonte;
    private $_listaNrCalculoFolha;

    public function __construct($entityManager, $hcmParameters, $dtPagamento){

        $this->entityManager = $entityManager;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_mes_ocorrencia = $hcmParameters->getDtCompetencia();
        $this->_dtPagamento = $dtPagamento;

        $this->_evento = new FpaeventomEntity(
            $entityManager, $hcmParameters, $this->_dtPagamento
        );
        $this->_calculo_folha_fonte = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_itens_calculo_fonte = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
    }

    private function _atribuiInicioPeriodoMedia($dtInicial, $dtFinal, $divisor){
        $dtInicialAux = DateUtil::subtraiIntervalo($dtFinal, $divisor, DateUtil::MES);
        
        if ($dtInicial > $dtInicialAux) {
            return $dtInicial;
        } else {
            return $dtInicialAux;
        }
    }

    private function _sumarizaVal($array){
        $getVal=function($row){
            return $row['VAL'];
        };
        $val=array_map($getVal,$array);
        return array_sum($val);
    }

    public function calculaMaiorMedia(
        $vinculo, $tipo_movimento, $lista_nr_eventos_origem, $dt_inicial,
        $dt_final, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        
        $fpaparamsindicato = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(
            array('nrorg' => $vinculo->getNrorg(), 'nrestruturam' => $vinculo->historico->getNrestrutsind()));
        
        if(is_object($fpaparamsindicato)){
            $divisor[1]=$fpaparamsindicato->getNrmesesmedia1();
            $divisor[2]=$fpaparamsindicato->getNrmesesmedia2();
            $divisor[3]=$fpaparamsindicato->getNrmesesmedia3();
        }
        else{
            MsgCalculo::getInstance()->adicionaMensagem('Não foram encontrados parâmetros de médias para sindicatos.');
        }

        $counter=3;
        $totmediaux=0;
        for ($i=1; $i<=$counter; $i++) {
            
            if($divisor[$i]==0){
                continue;
            }
            
            $dt_inicial = $this->_atribuiInicioPeriodoMedia($dt_inicial, $dt_final, $divisor[$i]);
            
            $media[$i] = $this->calcula($vinculo, $tipo_movimento, $lista_nr_eventos_origem, $dt_inicial,
                $dt_final, $nrTipoModalidadeCalc, $nrOcorrenciaCalc);
            
            $totmedia[$i] = $this->_sumarizaVal($media[$i]) / $divisor[$i];
            
            if($totmedia[$i] > $totmediaux){
                $totmediaux = $totmedia[$i];
                $mediaux = $media[$i];
            }
        }
        
        return $mediaux;
    }

    public function calcula(
        $vinculo, $tipo_movimento, $lista_nr_eventos_origem, $dt_inicial,
        $dt_final, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        $this->_listaNrCalculoFolha = $this->_calculo_folha_fonte->retornaCalculoFolhaPeriodo(
            $vinculo->getNrorg(), $tipo_movimento, $dt_inicial, $dt_final, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
        );
        
        if (empty($lista_nr_eventos_origem) === false) {
            $arrayNrCalculoFolha = explode(',',$this->_listaNrCalculoFolha);
            
            if($lista_nr_eventos_origem == '2010'){
                if(count($arrayNrCalculoFolha) > 1000){
                    $arrayRetornado = array();
                    $arrayNrCalculoFolhaDividos = array_chunk($arrayNrCalculoFolha, 1000, true);
                    
                    foreach ($arrayNrCalculoFolhaDividos as $arrayNrCalculoFolhaDivido) {
                        $stringNrCalculoFolha = implode(',', $arrayNrCalculoFolhaDivido);
                        
                        // Atualiza $arrayRetornado corretamente usando array_merge
                        $lista_de_eventos = array_merge(
                            $arrayRetornado,
                            $this->_itens_calculo_fonte->totalizaItensHistorico($vinculo, $stringNrCalculoFolha, $lista_nr_eventos_origem)
                        );

                        // Atualiza $arrayRetornado para o próximo loop
                        $arrayRetornado = $lista_de_eventos;
                    }
                
                    // Correção na condição e na lógica de soma
                    $totalEventos = sizeof($lista_de_eventos);
                    
                    for ($i = 0; $i < $totalEventos; $i++) {
                        $lista_eventos[$lista_nr_eventos_origem]["REF"] += $lista_de_eventos[$i]["REF"];
                        $lista_eventos[$lista_nr_eventos_origem]["VAL"] += $lista_de_eventos[$i]["VAL"];
                    }
                } else {
                    $lista_eventos  = $this->_itens_calculo_fonte->totalizaItensHistorico(
                        $vinculo, $this->_listaNrCalculoFolha, $lista_nr_eventos_origem
                    );
                }
            } else {
                $lista_eventos  = $this->_itens_calculo_fonte->totalizaItensHistorico(
                    $vinculo, $this->_listaNrCalculoFolha, $lista_nr_eventos_origem
                );
            }
            
            return $lista_eventos;
        } 
    }

    public function geraEventoMedia(
        $item_calculo_destino, $listaEventosTotalizados, $listaEventosOrigemDestino, $tipoMovimento,
        $ehMedia, $divisor = null, $tipoLancamento = null
    ) {
        //var_dump("divisor", $divisor);
        if ($divisor === null) {
            $aux = explode(',', $this->_listaNrCalculoFolha);
            $divisor = count($aux);
			if ($divisor>12){
		       $divisor = 12;
		    }
        }
        
        foreach ($listaEventosTotalizados as $nrEventoOrigem => $eventoTotalizado){
            $nrEventoDestino = $listaEventosOrigemDestino[$nrEventoOrigem];
            $evento = $this->_evento->retornaEvento($nrEventoDestino);

            $this->_evento->setMesCompetencia($this->_mes_ocorrencia);
            $this->_evento->setDataPagamento($this->_dtPagamento);
            
            if($divisor == 0) {
                $divisor = 1;
            }

            $total_ref = $eventoTotalizado[FpaitecalcfolhaEntity::REFERENCIA]/$divisor;
            $total_val = $eventoTotalizado[FpaitecalcfolhaEntity::VALOR]/$divisor;
            $calculoEvento = $eventoTotalizado[FpaitecalcfolhaEntity::VALOR].' / '.$divisor.'(divisor) = '.$total_val;
            
            if ($evento->historico->getIdmediavalref() === FpaeventohEntity::IDMEDIAVALREF_REFERENCIA) {
                
                if ($evento->historico->getIdunidrefereven() === FpaeventohEntity::IDUNIDREFEREVEN_HORA_MINUTO) {
                   $total_ref = FPA::converteDecimalHoraMinuto($total_ref);
                }
                
                $item_calculo_destino->geraEvento($evento, null, 0.001, $total_ref, 1, $tipoLancamento);
            } else {
                MemoriaCalculo::getInstance()->escreveEventos($item_calculo_destino->getNrcalculofolha(),$evento,$calculoEvento,$this->_mes_ocorrencia, $total_val, $total_ref,null,0,null,100);
                $item_calculo_destino->geraEvento($evento, null, $total_val, 0, 1, $tipoLancamento);
            }   
            
            if ($ehMedia === true){
                $posicao = $item_calculo_destino->procuraEvento($evento, null);
                if ($posicao !== -1) {
                    $item_calculo_destino->atribuiCalculoDemonstrativo(
                        $posicao, FpaitecalcfolhaEntity::IDEVENDEMONCALC_SIM
                    );
                }
            }
        }
    }
    
     public function retornaEventoMedia(
        $item_calculo_destino, $listaEventosTotalizados, $listaEventosOrigemDestino, $tipoMovimento,
        $ehMedia, $divisor = null, $tipoLancamento = null, $base = 0, $jornada_diaria
    ) {
        if ($divisor === null) {
            $aux = explode(',', $this->_listaNrCalculoFolha);
            $divisor = count($aux);
			if ($divisor>12){
		       $divisor = 12;
		    }
        }
        $valor = 0;
        
        foreach ($listaEventosTotalizados as $nrEventoOrigem => $eventoTotalizado){
            $nrEventoDestino = $listaEventosOrigemDestino[$nrEventoOrigem];
            $evento = $this->_evento->retornaEvento($nrEventoDestino);

            $this->_evento->setMesCompetencia($this->_mes_ocorrencia);
            $this->_evento->setDataPagamento($this->_dtPagamento);
            
            $total_ref = $eventoTotalizado[FpaitecalcfolhaEntity::REFERENCIA]/$divisor;
            $total_val = $eventoTotalizado[FpaitecalcfolhaEntity::VALOR]/$divisor;
            $calculoEvento = $eventoTotalizado[FpaitecalcfolhaEntity::VALOR].' / '.$divisor.'(divisor) = '.$total_val;
            
            if ($evento->historico->getIdmediavalref() === FpaeventohEntity::IDMEDIAVALREF_REFERENCIA) {
                
                if ($evento->historico->getIdunidrefereven() === FpaeventohEntity::IDUNIDREFEREVEN_HORA_MINUTO) {
                   $total_ref = FPA::converteDecimalHoraMinuto($total_ref);
                }
                
                $valor += $item_calculo_destino->retornaValorEvento($evento, null, 0, $total_ref, 1, $tipoLancamento, null, $base, $jornada_diaria);
            } else {
                $valor += $item_calculo_destino->retornaValorEvento($evento, null, $total_val, 0, 1, $tipoLancamento);
            }
            
            
        }
        return $valor;
    }
}