<?php

namespace HCM\FPA\Calculos\PLR;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpaeventohEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;
use \Setforms2\Shared\ORM\DateTime;

class CalculoPLR extends Calculo {
    
    public $periodoInicioNovoMetodoPLR;
    public $periodoFinalNovoMetodoPLR;
    public $avosDireitoPLR;
    public $valorPLR;
    public $valor1avoPLR;
    public $tipoPeriodoPLR;

    public function calculaPLR($vinculo, $tp_calc_mov, $contador, $dataSet = null){
        
        if($this->dataSet['DSORDEM'] == "LISTA_LOTE"){
            if(!is_array($this->dataSet['DSINICIO'])){
                $nrlotevincs = explode(',',$this->dataSet['DSINICIO']);
            } else {
                $nrlotevincs = $this->dataSet['DSINICIO'];
            }
            
            foreach($nrlotevincs as $nrlotevinc){
                $vinculoEncontrado = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findOneBy(array('nrorg' => $this->_nrorg, 'nrlotevinc' =>  $nrlotevinc, 'nrvinculom' => $vinculo->getNrvinculom()));

                if(!empty($vinculoEncontrado)){
                    $this->_nrlotevinc = $nrlotevinc;
                    continue;
                }
            }
        }
        
        $calcula = $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        
        if(!$calcula){
            return;
        }
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }
        $liquido = $this->_controleLiquido();
        if ($liquido == true) {
            MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.', MsgCalculo::ERRO);
            return;
        }
        
        $novoMetodoPLR = false;
        if(!empty($dataSet) && !empty($dataSet['VRPLR'])){
            $periodoPLR = !empty($dataSet['PERIODOPLR']) ? $dataSet['PERIODOPLR'] : null;
            
            //% de desconto sobre o PLR caso encontre mais que a quantidade maxima faltas/faltas injustificadas ou atestados
            $descFaltaJust = !empty($dataSet['VRDESCFALTAJUST']) ? $dataSet['VRDESCFALTAJUST'] : null;
            $qtDiasMaximoJust = !empty($dataSet['QTDIASMAXIMOJUST']) ? $dataSet['QTDIASMAXIMOJUST'] : null;;
            
            $descFaltaInjust = !empty($dataSet['VRDESCFALTAINJUST']) ? $dataSet['VRDESCFALTAINJUST'] : null;
            $qtDiasMaximoInjust = !empty($dataSet['QTDIASMAXIMOINJUST']) ? $dataSet['QTDIASMAXIMOINJUST'] : null;
            
            $descAtestMed = !empty($dataSet['VRDESCATESTMED']) ? $dataSet['VRDESCATESTMED'] : null;
            $qtDiasMaximoAtest = !empty($dataSet['QTDIASMAXIMOATEST']) ? $dataSet['QTDIASMAXIMOATEST'] : null;
            
            $descEventoPeri = !empty($dataSet['VRDESCEVENPERI']) ? $dataSet['VRDESCEVENPERI'] : null;
            $qtDiasMaximoEvento = !empty($dataSet['QTDIASMAXIMOEVENT']) ? $dataSet['QTDIASMAXIMOEVENT'] : null;
            $qtHorasMaximoEvento = !empty($qtDiasMaximoEvento) ?  $qtDiasMaximoEvento * $this->_jornada_diaria : null;
            
            $descSituPeri = !empty($dataSet['VRDESCSITUPERI']) ? $dataSet['VRDESCSITUPERI'] : null;
            $qtDiasMaximoSitu = !empty($dataSet['QTDIASMAXIMOSITU']) ? $dataSet['QTDIASMAXIMOSITU'] : null;
            
            if(!empty($periodoPLR)){
                $msg = $this->calculaPeriodoInicialeFinalNovoMetodoPLR($periodoPLR, $dataSet, $vinculo);
            }
            
            if(!empty($msg)){
                if($msg == 'Sem data base sindical no sindicato do vínculo.'){
                    MsgCalculo::getInstance()->adicionaMensagem('Sindicato do vínculo não possui data base informada, impossibilitando o cálculo de PLR para o tipo de período PLR escolhido.', MsgCalculo::ERRO);
                    return;
                }
                
                if($msg == 'Vínculo não possui Sindicato.'){
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem sindicato, impossibilitando o cálculo de PLR para o tipo de período PLR escolhido.', MsgCalculo::ERRO);
                    return;
                }
                
                if($msg == 'Vinculo com admissão posterior ao periodo de apuração da Data Base do Sindicato.'){
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo com admissão posterior ao período de apuração da Data Base do Sindicato.', MsgCalculo::ERRO);
                    return;
                }
                
                if($msg == 'Vinculo com rescisão anterior ao periodo de apuração da Data Base do Sindicato.'){
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo com rescisão anterior ao período de apuração da Data Base do Sindicato.', MsgCalculo::ERRO);
                    return;
                }
            }
            
            if($dataSet['IDPROPMESEMPRESA'] == 'S' && !empty($periodoPLR)){
                $qtMinimoDias = !empty($dataSet['QTDIASTRABMES']) ? $dataSet['QTDIASTRABMES'] : null;
                $nrSituAfastamentos = !empty($dataSet['NRSITUFUNCMAFAST_LIST']) ? $dataSet['NRSITUFUNCMAFAST_LIST'] : null;

                $this->avosDireitoPLR = $this->calculaAvosPLR($vinculo, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR, $qtMinimoDias, $periodoPLR, $nrSituAfastamentos);
            }
            
            $novoMetodoPLR = true;
        }
        
        $this->_controleLiquidoParcial();
        
        $this->_inicializaVariaveisRegra();

        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        $this->_verificaTrabalhoEfetivo($vinculo);
        
        MemoriaCalculo::getInstance()->criaCabecalho($this->entityManager, $this->hcmParameters, $this->_vinculo, $this->_nr_tipo_calculo, $this->_sit_func_resumo, $this->_salario, $this->_dias_uteis, $this->_dias_rsr,
            $this->_mes_competencia, $this->_dt_pagamento, $this->_nrOcorrenciaCalculo, $this->_obj_tipo_calculo->getNmtipocalculo(), $this->_gozo_ferias, null, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR,
            $this->tipoPeriodoPLR); 
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        //Metodo novo de calculo de PLR para que se tiver VRPLR lançar o evento 870 de forma automatica e calcular.
        if($novoMetodoPLR == true){
            
            $atestadosNoPeriodo = null;
            $faltasJustificadasNoPeriodo = null;
            $faltasInjustificadasNoPeriodo = null;
            $reembolsoFaltasIndevidas = null;
            
            if($this->avosDireitoPLR == 0 && $dataSet['IDPROPMESEMPRESA'] == 'S' && !empty($periodoPLR)){
                MsgCalculo::getInstance()->adicionaMensagem('Vinculo não possui direito ao PLR pois não possui avós no periodo.', MsgCalculo::ERRO);
                return;
            }
            
            $eventomPLR = $this->retornaEvento(FpatipoassoceveEntity::ASSOC_PLR);
            
            if(empty($periodoPLR)){
                $valorPL = $dataSet['VRPLR'];
                $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                $this->valorPLR = (float)$valorPL;

                MemoriaCalculo::getInstance()->detalhamentoPLR($this->valorPLR); 
            } else {
                $atestadosNoPeriodo = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), 810, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR);
                $faltasJustificadasNoPeriodo = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), '490,495', $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR);
                $faltasInjustificadasNoPeriodo = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), 1040, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR);
                $reembolsoFaltasIndevidas = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), 36, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR);
                
                $this->valorPLR = $this->valor1avoPLR * $this->avosDireitoPLR;

                MemoriaCalculo::getInstance()->detalhamentoPLR($this->valorPLR, $this->valor1avoPLR, $this->avosDireitoPLR);
            }

            $diasFaltasJustificada = 0;
            $diasFaltasInjustificada = 0;
            $diasAtestados = 0;
    
            $descontoAtestado = 0;
            $descontoJustificado = 0;
            $descontoInjustificado = 0;
            $descontoEventos = 0;
            $descontoSitu = 0;

            if(!empty($dataSet['NREVENTO_LIST']) && !empty($periodoPLR)){
                foreach($dataSet['NREVENTO_LIST'] as $key => $evento){
                    $diasEventos = 0;
                    $horasEventos = 0;
                    $eventosNoPeriodo = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculoNormal($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), $evento, $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR, $this->hcmParameters->getDtCompetencia());
                    
                    foreach($eventosNoPeriodo as $eventoNoPeriodo){
                        if($eventoNoPeriodo['IDUNIDREFEREVEN'] == 'DIA'){
                            $diasEventos += $eventoNoPeriodo['VRREFFOLHA'];
                        } else {
                            $horasEventos += $eventoNoPeriodo['VRREFFOLHA'];
                        }
                    }
                    
                    if(($diasEventos > $qtDiasMaximoEvento) || ($horasEventos > $qtHorasMaximoEvento)){
                        $descontoEventos += $this->valorPLR * ($descEventoPeri/100);
                    }
                    MemoriaCalculo::getInstance()->detalhamentoDescontoDiretoPLR($evento, $dataSet['NMEVENTOH'][$key], $diasEventos, $horasEventos);
                }
            }
            
            if(!empty($dataSet['NRSITUFUNCM_LIST']) && !empty($periodoPLR)){
                foreach($dataSet['NRSITUFUNCM_LIST'] as $key => $situfunc){
                    $diasSituFunc = 0;

                    $situsNoPeriodo = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNrsituDentroPeriodo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), $this->periodoInicioNovoMetodoPLR, $this->periodoFinalNovoMetodoPLR, $situfunc);

                    foreach($situsNoPeriodo as $situNoPeriodo){
                        $diasSituFunc += $situNoPeriodo->getNrqtdiasitufunc();
                    }
                    
                    if($diasSituFunc > $qtDiasMaximoSitu){
                        $descontoSitu += $this->valorPLR * ($descSituPeri/100);
                    }
                    MemoriaCalculo::getInstance()->detalhamentoSituFuncPLR($situfunc, $dataSet['NMSITUFUNCH'][$key], $diasSituFunc);
                }
            }

            if(!empty($atestadosNoPeriodo) && !empty($periodoPLR)){
                foreach($atestadosNoPeriodo as $atestadoNoPeriodo){
                    $diasAtestados += $atestadoNoPeriodo->getVrreffolha();
                }
                
                if($diasAtestados > $qtDiasMaximoAtest){
                    $descontoAtestado = $this->valorPLR * ($descAtestMed/100);
                }
                MemoriaCalculo::getInstance()->detalhamentoAtestadosPLR($diasAtestados);
            }

            if(!empty($faltasJustificadasNoPeriodo) && !empty($periodoPLR)){
                foreach($faltasJustificadasNoPeriodo as $faltaJustificadaNoPeriodo){
                    $diasFaltasJustificada += $faltaJustificadaNoPeriodo->getVrreffolha();
                }
                
                if($diasFaltasJustificada > $qtDiasMaximoJust){
                    $descontoJustificado = $this->valorPLR * ($descFaltaJust/100);
                }
                MemoriaCalculo::getInstance()->detalhamentoFaltasJustificadasPLR($diasFaltasJustificada);
            }
            
            if(!empty($faltasInjustificadasNoPeriodo) && !empty($periodoPLR)){
                foreach($faltasInjustificadasNoPeriodo as $faltaInjustificadaNoPeriodo){
                    $diasFaltasInjustificada += $faltaInjustificadaNoPeriodo->getVrreffolha();
                }
                
                if(!empty($reembolsoFaltasIndevidas)){
                    foreach($reembolsoFaltasIndevidas as $reembolsoFaltaIndevida){
                        $diasFaltasInjustificada -= $reembolsoFaltaIndevida->getVrreffolha();
                    }
                }
                
                if($diasFaltasInjustificada < 0){
                    $diasFaltasInjustificada = 0;
                }

                if($diasFaltasInjustificada > $qtDiasMaximoInjust){
                    $descontoInjustificado = $this->valorPLR * ($descFaltaInjust/100);
                }
                MemoriaCalculo::getInstance()->detalhamentoFaltasInjustificadasPLR($diasFaltasInjustificada);
            }

            $valorPLRold = $this->valorPLR;
            $this->valorPLR = ((((($this->valorPLR - $descontoInjustificado) - $descontoJustificado) - $descontoAtestado) - $descontoEventos) - $descontoSitu);

            MemoriaCalculo::getInstance()->detalhamentoFinalPLR($this->valorPLR, $valorPLRold, $descontoInjustificado, $descontoJustificado, $descontoAtestado, $descontoEventos, $descontoSitu);
            
            if($this->valorPLR <= 0){
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem direito a PLR,por possuir liquido PLR negativo.', MsgCalculo::ERRO);
                return;
            }
            
            $this->_itens_calculo_principal->geraEvento($eventomPLR, $this->hcmParameters->getDtCompetencia(), $this->valorPLR, 0, 1);
        }
        
        $this->_executaPLR($vinculo, $this->_nr_tipo_calculo, $tp_calc_mov);
        
        $this->_geraPensao();
            
        $this->_calculaTotais();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        MemoriaCalculo::getInstance()->setMovimentos();
        MemoriaCalculo::getInstance()->setDetalhamentoCalculoAvoPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoAvosPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoFaltasJustificadasPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoFaltasInjustificadasPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoAtestadosPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoDescontoDiretoPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoSituFuncPLR();
        MemoriaCalculo::getInstance()->setDetalhamentoFinalPLR();
        
        $arquivoMemoria = MemoriaCalculo::getInstance()->getNmArquivo();
        
        $lista = null;
        
        if($arquivoMemoria){
            $lista['ARQUIVOMEMORIA'] = FPA::getTempDirectory().'/'.$arquivoMemoria;
        }
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();

        MemoriaCalculo::getInstance()->clearDataPLR();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo(null, $this->_nrlotevinc);
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }
    
    public function retornaEvento($assoc){
        return $this->_eventom->retornaEvento($assoc);
    }
    
    public function calculaPeriodoInicialeFinalNovoMetodoPLR($periodoPLR, $dataSet, $vinculo){
        if(!empty($dataSet) && $dataSet['PERIODOPLR']){
            switch($dataSet['PERIODOPLR']){
                case '6NORM': {
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $dtmescompetenc = $mesCompetencia;
                    $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc,DateUtil::FORMATO_BRASILEIRO, true);
                    $periodoIni = $dtmescompetenc->sub(new \DateInterval('P6M'));
                    $periodoFim = DateUtil::getDataDeString($mesCompetencia,DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P1D'));

                    $this->periodoInicioNovoMetodoPLR = $periodoIni->format('d/m/Y');
                    $this->periodoFinalNovoMetodoPLR = $periodoFim->format('d/m/Y');
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));

                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }

                    $this->avosDireitoPLR = 6;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 6 Meses.'.PHP_EOL.'A partir da data: '.substr($dtmescompetenc,0,10).' será retrocedido os últimos 6 Meses, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;
                }
                case '12NORM':{
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $dtmescompetenc = $mesCompetencia;
                    $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc,DateUtil::FORMATO_BRASILEIRO, true);
                    $periodoIni = $dtmescompetenc->sub(new \DateInterval('P12M'));
                    $periodoFim = DateUtil::getDataDeString($mesCompetencia,DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P1D'));
                    
                    $this->periodoInicioNovoMetodoPLR = $periodoIni->format('d/m/Y');
                    $this->periodoFinalNovoMetodoPLR = $periodoFim->format('d/m/Y');
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));
                    
                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }

                    $this->avosDireitoPLR = 12;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 12 Meses.'.PHP_EOL.'A partir da data: '.substr($dtmescompetenc,0,10).' será retrocedido os últimos 12 Meses, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;
                }
                case '6SEMESTREANT':{
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $dtmescompetenc = $mesCompetencia;
                    $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc,DateUtil::FORMATO_BRASILEIRO, true);
                    $primeiroDiaSemestreAnt = DateUtil::calcularDiaSemestreAnterior($dtmescompetenc, true);
                    $UltimoDiaSemestreAnt = DateUtil::calcularDiaSemestreAnterior($dtmescompetenc, false);
        
                    $periodoIni = DateUtil::getDataDeString($primeiroDiaSemestreAnt,DateUtil::FORMATO_BRASILEIRO, true);
                    $periodoFim = DateUtil::getDataDeString($UltimoDiaSemestreAnt,DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $this->periodoInicioNovoMetodoPLR = $periodoIni->format('d/m/Y');
                    $this->periodoFinalNovoMetodoPLR = $periodoFim->format('d/m/Y');
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));

                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }

                    $this->avosDireitoPLR = 6;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 6 meses (Semestre Anterior ao Cálculo).'.PHP_EOL.'A partir da data: '.substr($dtmescompetenc,0,10).' será verificado o semestre anterior ao cálculo e analisado o semestre completo, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;
                }
                case '12SEMESTREANT':{
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $dtmescompetenc = $mesCompetencia;
                    $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc,DateUtil::FORMATO_BRASILEIRO, true);
                    $primeiroDiaSemestreAnt = DateUtil::calcularDiaSemestreAnterior($dtmescompetenc, true);
                    $UltimoDiaSemestreAnt = DateUtil::calcularDiaSemestreAnterior($dtmescompetenc, false);
        
                    $periodoIni = DateUtil::getDataDeString($primeiroDiaSemestreAnt,DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P6M'));
                    $periodoFim = DateUtil::getDataDeString($UltimoDiaSemestreAnt,DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $this->periodoInicioNovoMetodoPLR = $periodoIni->format('d/m/Y');
                    $this->periodoFinalNovoMetodoPLR = $periodoFim->format('d/m/Y');
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));
                    
                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }

                    $this->avosDireitoPLR = 12;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 12 meses (Semestre Anterior ao Cálculo).'.PHP_EOL.'A partir da data: '.substr($dtmescompetenc,0,10).' será verificado dois semestres anteriores ao cálculo e analisado os dois semestres completos, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;
                }
                case '6ANTMESDATABASE':{
                    if(empty($vinculo->historico->getNrestrutsind())){
                        return "Vínculo não possui Sindicato.";
                    }
                    
                    $dataBaseSind = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->getParamSindicato($vinculo->historico->getNrestrutsind(), $this->hcmParameters->getNrorgTrabalho(),$this->hcmParameters->getDtCompetencia())->getDtbasesindical();
                    
                    if(empty($dataBaseSind)){
                        return "Sem data base sindical no sindicato do vínculo.";
                    }
                    
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $anoCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia())->format('Y');
                    $mesDataBaseSind = $dataBaseSind->format('m');
                    
                    $databaseAnoCorrente = new DateTime($anoCompetencia . '-' . $mesDataBaseSind . '-1');
                    $databaseAnoCorrenteUltimoDiaMesAnterior = new DateTime($anoCompetencia . '-' . $mesDataBaseSind . '-1');
                    $databaseAnoCorrenteUltimoDiaMesAnterior = $databaseAnoCorrenteUltimoDiaMesAnterior->sub(new \DateInterval('P1D'));
                    
                    $periodoIni = $databaseAnoCorrente->sub(new \DateInterval('P6M'))->format('d/m/Y');
                    $periodoFim = $databaseAnoCorrenteUltimoDiaMesAnterior->format('d/m/Y');
                    
                    $this->periodoInicioNovoMetodoPLR = $periodoIni;
                    $this->periodoFinalNovoMetodoPLR = $periodoFim;
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc());
                    
                    if($admissaoVinc >= DateUtil::truncateData(DateUtil::getDataDeString($periodoFim))){
                        return "Vinculo com admissão posterior ao periodo de apuração da Data Base do Sindicato.";
                    }
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));
                    
                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());
                        
                        if($rescisaoVinc <= DateUtil::truncateData(DateUtil::getDataDeString($periodoIni))){
                            return "Vinculo com rescisão anterior ao periodo de apuração da Data Base do Sindicato.";
                        }
                        
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }

                    $this->avosDireitoPLR = 6;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 6 meses (Mês Anterior ao Mês da Data Base).'.PHP_EOL.'A partir da data: '.substr($databaseAnoCorrente->format('d/m/Y'),0,10).' será verificado o semestre anterior a data base do sindicato e analisado o semestre completo, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;                    
                }
                case '12ANTMESDATABASE':{
                    if(empty($vinculo->historico->getNrestrutsind())){
                        return "Vínculo não possui Sindicato.";
                    }
                    
                    $dataBaseSind = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->getParamSindicato($vinculo->historico->getNrestrutsind(), $this->hcmParameters->getNrorgTrabalho(),$this->hcmParameters->getDtCompetencia())->getDtbasesindical();
                    
                    if(empty($dataBaseSind)){
                        return "Sem data base sindical no sindicato do vínculo.";
                    }
                    
                    $mesCompetencia = $this->hcmParameters->getDtCompetencia();
                    $anoCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia())->format('Y');
                    $mesDataBaseSind = $dataBaseSind->format('m');
                    
                    $databaseAnoCorrente = new DateTime($anoCompetencia . '-' . $mesDataBaseSind . '-1');
                    $databaseAnoCorrenteUltimoDiaMesAnterior = new DateTime($anoCompetencia . '-' . $mesDataBaseSind . '-1');
                    $databaseAnoCorrenteUltimoDiaMesAnterior = $databaseAnoCorrenteUltimoDiaMesAnterior->sub(new \DateInterval('P1D'));
                    
                    $periodoIni = $databaseAnoCorrente->sub(new \DateInterval('P12M'))->format('d/m/Y');
                    $periodoFim = $databaseAnoCorrenteUltimoDiaMesAnterior->format('d/m/Y');
                    
                    $this->periodoInicioNovoMetodoPLR = $periodoIni;
                    $this->periodoFinalNovoMetodoPLR = $periodoFim;
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc());
                    
                    if($admissaoVinc >= DateUtil::truncateData(DateUtil::getDataDeString($periodoFim))){
                        return "Vinculo com admissão posterior ao periodo de apuração da Data Base do Sindicato.";
                    }
                    
                    $admissaoVinc = DateUtil::getDataDeString($vinculo->getDtadmissaovinc()->format('01/m/Y'));
                    
                        //01/02             31/07        01/02              01/02
                    if($admissaoVinc <= $periodoFim && $admissaoVinc >= $periodoIni){
                        $this->periodoInicioNovoMetodoPLR = $admissaoVinc->format('d/m/Y');
                    }
                    
                    if(!is_null($vinculo->getDtrescisaovinc())){
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());
                        
                        if($rescisaoVinc <= DateUtil::truncateData(DateUtil::getDataDeString($periodoIni))){
                            return "Vinculo com rescisão anterior ao periodo de apuração da Data Base do Sindicato.";
                        }
                        
                        #20/06
                        $rescisaoVinc = DateUtil::getDataDeString($vinculo->getDtrescisaovinc()->format('01/m/Y'));
                        
                            // 01/06            31/07           01/06           01/02
                        if($rescisaoVinc <= $periodoFim && $rescisaoVinc >= $periodoIni){
                            $this->periodoFinalNovoMetodoPLR = $rescisaoVinc->format('d/m/Y');
                        }
                    }
                    
                    $this->avosDireitoPLR = 12;
                    $valorPL = $dataSet['VRPLR'];
                    $valorPL = str_replace(",", ".", $valorPL); // Substitui a vírgula por ponto
                    $this->valorPLR = (float)$valorPL;
                    $this->valor1avoPLR = $this->valorPLR/$this->avosDireitoPLR;
                    $this->tipoPeriodoPLR = 'Foi selecionado o período de Últimos 12 meses (Mês Anterior ao Mês da Data Base).'.PHP_EOL.'A partir da data: '.substr($databaseAnoCorrente->format('d/m/Y'),0,10).' será verificado os dois semestres anteriores a data base do sindicato e analisado os semestres completos, ou seja, De: '.$this->periodoInicioNovoMetodoPLR.' Até: '.$this->periodoFinalNovoMetodoPLR;
                    MemoriaCalculo::getInstance()->detalhamentoCalculoAvoPLR($this->valor1avoPLR, $this->valorPLR, $this->avosDireitoPLR);
                    break;
                }
            }
        }
    }
    
    private function calculaAvosPLR($vinculo, $dt_inicio, $dt_fim, $qtMinimoDias, $periodoPLR, $nrSituAfastamentos) {
        $dt_inicio = DateUtil::truncateData(DateUtil::getDataDeString($dt_inicio));
        $dt_fim = DateUtil::truncateData(DateUtil::getDataDeString($dt_fim));
        $nrAvos = 0;
        MemoriaCalculo::getInstance()->detalhamentoAvosPLR($vinculo->getNmvinculom());
        
        while ($dt_inicio <= $dt_fim) {
            $dt_mes_inicio = clone $dt_inicio;
            $mes = $dt_mes_inicio->format('m');
            $ano = $dt_mes_inicio->format('Y');
            $dt_mes_inicio->setDate($ano, $mes, 1);
    
            $dt_mes_fim = clone $dt_mes_inicio;
            $dt_mes_fim->setDate($ano, $mes, $dt_mes_inicio->format('t'));
    
            // Verifica se há admissão dentro do período do mês
            if (!is_null($vinculo->getDtadmissaovinc()) && $vinculo->getDtadmissaovinc() >= $dt_mes_inicio && $vinculo->getDtadmissaovinc() <= $dt_mes_fim) {
                $dt_mes_inicio = clone $vinculo->getDtadmissaovinc();

                MemoriaCalculo::getInstance()->detalhamentoAvosPLR(null, substr($vinculo->getDtadmissaovinc(),0,10));
            }
    
            // Verifica se há rescisão dentro do período do mês
            if (!is_null($vinculo->getDtrescisaovinc()) && $vinculo->getDtrescisaovinc() >= $dt_mes_inicio && $vinculo->getDtrescisaovinc() <= $dt_mes_fim) {
                $dt_mes_fim = clone $vinculo->getDtrescisaovinc();

                MemoriaCalculo::getInstance()->detalhamentoAvosPLR(null, null, substr($vinculo->getDtrescisaovinc(),0,10));
            }
    
            // Verifica se o período do mês é válido
            $dias_intervalo = DateUtil::getQtdeDiasEntre($dt_mes_inicio, $dt_mes_fim) + 1;
            if ($dias_intervalo >= $qtMinimoDias) {
                $dias_afastados = 0;
    
                foreach ($nrSituAfastamentos as $nrSituAfastamento) {
                    $dias_afastados += $this->quantidadeDiasAfastados($vinculo, $dt_mes_inicio, $dt_mes_fim, $nrSituAfastamento);
                }
    
                $dias_faltas = $this->quantidadeFaltas($vinculo, $dt_mes_inicio, $dt_mes_fim);
    
                if (($dias_intervalo - $dias_afastados - $dias_faltas) >= $qtMinimoDias) {
                    $nrAvos += 1;
                }
            }
            MemoriaCalculo::getInstance()->detalhamentoAvosPLR(null, null, null, substr($dt_mes_inicio,0,10), substr($dt_mes_fim,0,10), $dias_afastados, $dias_faltas, $nrAvos);

            // Avança para o próximo mês
            $dt_inicio->modify('+1 month');
        }
    
        return $nrAvos;
    }
    
    private function quantidadeDiasAfastados($vinculo, $dt_inicio, $dt_fim, $nrSituAfastamento){
        $dt_inicio = DateUtil::truncateData($dt_inicio);
        $dt_fim = DateUtil::truncateData($dt_fim);
        $situsNoPeriodo = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNrsituEntrePeriodo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), $dt_inicio, $dt_fim, $nrSituAfastamento);
        $dias_afastados = 0;

        foreach($situsNoPeriodo as $situNoPeriodo){

            if($situNoPeriodo->getDtinisitufunc() >= $dt_inicio && $situNoPeriodo->getDtfimsitufunc() <= $dt_fim){
                $dias_afastados += count($this->_alteracao_situacao_funcional->retornaDiasAfastamento(
                    $situNoPeriodo->getNrsitufuncm(), $situNoPeriodo->getDtinisitufunc(), is_null($situNoPeriodo->getDtfimsitufunc()) ? $dt_fim : $situNoPeriodo->getDtfimsitufunc(), false
                ));
            } else if($situNoPeriodo->getDtinisitufunc() < $dt_inicio && ($situNoPeriodo->getDtfimsitufunc() > $dt_fim || is_null($situNoPeriodo->getDtfimsitufunc()))){
                $dias_afastados += count($this->_alteracao_situacao_funcional->retornaDiasAfastamento(
                    $situNoPeriodo->getNrsitufuncm(), $dt_inicio, $dt_fim, false
                ));
            } else if($situNoPeriodo->getDtinisitufunc() >= $dt_inicio && $situNoPeriodo->getDtinisitufunc() <= $dt_fim && ($situNoPeriodo->getDtfimsitufunc() > $dt_fim || is_null($situNoPeriodo->getDtfimsitufunc()))){
                $dias_afastados += count($this->_alteracao_situacao_funcional->retornaDiasAfastamento(
                    $situNoPeriodo->getNrsitufuncm(), $situNoPeriodo->getDtinisitufunc(), $dt_fim, false
                ));
            } else if($situNoPeriodo->getDtinisitufunc() < $dt_inicio && ($situNoPeriodo->getDtfimsitufunc() <= $dt_fim && $situNoPeriodo->getDtfimsitufunc() >= $dt_inicio)){
                $dias_afastados += count($this->_alteracao_situacao_funcional->retornaDiasAfastamento(
                    $situNoPeriodo->getNrsitufuncm(), $dt_inicio, is_null($situNoPeriodo->getDtfimsitufunc()) ? $dt_fim : $situNoPeriodo->getDtfimsitufunc(), false
                ));
            }
        }

        return $dias_afastados;
    }
    
    private function quantidadeFaltas($vinculo, $dt_inicio, $dt_fim){
        $dt_inicio = DateUtil::truncateData($dt_inicio);
        $dt_fim = DateUtil::truncateData($dt_fim);
        
        $lista_eventos = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodoVinculo($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), '1040,1041,1043,1044,36', $dt_inicio, $dt_fim);

        $total_dias = 0;

        foreach ($lista_eventos as $nr_evento=> $item_evento) {
            $evento = $this->retornaEvento($item_evento->getNreventom());

            $tipo_referencia = $evento->historico->getIdunidrefereven();
            $id_faltas_ferias = $evento->historico->getIdfaltasferias();

            $total_aux = number_format(
                FpaeventomEntity::compatibilizaReferenciaDias($item_evento->getVrreffolha(), $tipo_referencia, $this->_jornada_diaria), 2
            );

            switch ($id_faltas_ferias){
                case FpaeventohEntity::IDFALTASFERIAS_SOMA:		$total_dias += $total_aux; break;
                case FpaeventohEntity::IDFALTASFERIAS_SUBTRAI: 	$total_dias -= $total_aux; break;
            }
        }

        return  floor($total_dias);
    }

}