<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\EventoService;
use HCM\FPA\Services\RelatCustomEventoService;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class EventoController extends Crud {

    protected $eventoService;
    protected $dataSourceName = 'getEvento';
    protected $parameterBagColumns = array('NRORG', 'NRORG_PADRAO','DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EventoService $eventoService, RelatCustomEventoService $relatCustomEventoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->eventoService = $eventoService;
        $this->relatCustomEventoService = $relatCustomEventoService;
        parent::__construct($dataSourceManager);
    }

    public function getEvento (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getEventoAtivo (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('fpa_evento_ativo_get');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function deleteEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->eventoService->deleteEvento($dataset);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $error;
            if(strpos($e->getMessage(), 'FOLHA.FKFPA_00006FPA_EVENT') !== false){
                if($this->eventoService->getRegrasEvento($dataset['NREVENTOM'], $dataset['NRORG'])){
                    $error = 'Atenção! Há incidências e regras cadastradas à esse evento! Primeiro as exclua para posteriormente conseguir excluir o evento.';  
                } else {
                    $error = 'Atenção! Há incidências cadastradas à esse evento! Primeiro as exclua para posteriormente conseguir excluir o evento.';  
                }
            } else if(strpos($e->getMessage(), 'FKFPA_00078FPA_EVENT') !== false){
                $error = 'Atenção! Há regras cadastradas à esse evento! Primeiro as exclua para posteriormente conseguir excluir o evento.';  
            } else if(strpos($e->getMessage(), 'FOLHA.FPA_EVENTOCTBL_FPA_EVENTOM_FK1') !== false){
                $error = 'Atenção! Há associações contábeis cadastradas associadas à esse evento! Primeiro as exclua para posteriormente conseguir excluir o evento.';  
            }
            $response->setError(new DTO\Response\Error(empty($error) ? $e->getMessage() : $error, empty($error) ? $e->getCode() : 5));
        }  
    }
    
    public function salvaAssocEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $returnOper = null;
            $returnAdm = null;
                   
            if($dataset['CDCONTCTBLDEBOPER'] || $dataset['CDHISTPADRDEBOPER'] || $dataset['CDCONTCTBLCREOPER'] || $dataset['CDHISTPADRCREOPER'] || $dataset['CDDESPADMFATOPER']){
                $returnOper = $this->eventoService->salvaAssocEventoOper($dataset);
            }
            if($dataset['CDCONTCTBLDEBADM'] || $dataset['CDHISTPADRDEBADM'] || $dataset['CDCONTCTBLCREADM'] || $dataset['CDHISTPADRCREADM'] || $dataset['CDDESPADMFATADM']){
                $returnAdm = $this->eventoService->salvaAssocEventoAdm($dataset);
            }
            if($returnAdm && $returnOper){
                $response->addDataSet(new DataSet("nrEvento", array("NREVENTOCTBL" => array($returnOper, $returnAdm))));
            } else if($returnAdm && !$returnOper){
                $response->addDataSet(new DataSet("nrEvento", array("NREVENTOCTBL" => $returnAdm)));
            } else if(!$returnAdm && $returnOper){
                $response->addDataSet(new DataSet("nrEvento", array("NREVENTOCTBL" => $returnOper)));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    
    public function cadAutoEvento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->eventoService->cadAutoEvento($row, $dataset);
            }
            $response->addNotification(new DTO\Response\Notification('cadEvento'), DTO\Response\Notification::TYPE_SUCCESS);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getParametrizacaoEsocialEvento(DTO\Request\Filter $request, DTO\Response $response){
        //Função similiar à 'getParametrizacaoEsocial' existente no RelatCustomEventoController, com diferença de apresentar mais uma condição,
        //adicionada para filtrar pelo campo NREVENTOM
        try{
           set_time_limit(0); 
           $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
           $parameters = array();
           foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO', 'P_NREVENTOM'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->relatCustomEventoService->getParametrizacaoEsocial($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getParametrizacaoEsocial', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function getEventoh($nreventom, $nreventoh){
        return $this->eventoService->getEventoh($nreventom, $nreventoh);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }

    public function replicacaoEvent(DTO\Request\DataSet $request, DTO\Response $response){
        $dataSet = $request->getDataSet()->getRows()[0];
        
        try{
            $tpMsg = DTO\Response\Message::TYPE_MESSAGE;

            $resultado = $this->eventoService->replicacaoEvent($dataSet);

            if($resultado){
                $msgRetorno = 'Replicação finalizada com sucesso!';
            }
            else {
                $msgRetorno = 'Ocorreu um erro na replicação. Tente novamente.';
                $tpMsg = DTO\Response\Message::TYPE_ERROR;
            }
            
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msgRetorno, $tpMsg));

        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
}