<?php
    namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico;

    use Doctrine\ORM\EntityManager;
    use HCM\Util\HCMParameters;
    use HCM\Util\DateUtil;
    use HCM\Util\Repositories;
    use HCM\Util\NovoCodigo;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro0HoleriteSantander;
    
class FpaHoleritemagnet
{
    protected $items = array();
    protected $removedItems = array();
    private $_pointer = 0;
        
    public function __construct($row, HCMParameters $hcmParameters,EntityManager $entityManager){
        $this->hcmParameters =  $hcmParameters;
        $this->entityManager =  $entityManager;
        
        $this->fpaHoleriteMag = Repositories::FPA_HOLERITEMAGNET;
    }
    
    public function addItem($item, $key = null)
    {
        if ($key === null) {
            $key = $this->_pointer++;
        }

        $this->items[$key] = $item;
        return $this;
    }


    public function getItems()
    {
        return $this->items;
    }


    public function getItem($key)
    {
        return $this->items[$key];
    }


    public function removeItem($object)
    {
        $items = $this->items;
        foreach ($items as $key => $item) {
            if ($item === $object) {
                unset($this->items[$key]);
                $this->removedItems[$key] = $object;
            }
        }
    }
    
    public function persisteItems()
    {   
        foreach ($this->items as $item) {
            $this->entityManager->persist($item);
            $this->entityManager->flush($item);
        }
    }

    public function retornaRegistros($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento){
        return $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findBy(array("mescompetencia"=>$mesCompetencia,"nrorg"=>$nrOrg,"cdbanco"=>$cdBanco,"nrtipomoviment"=>$nrTipoMovimento),array("cdinscestrut"=>"asc", "nrvinculom"=>"asc", "idtiporegistro"=>"asc", "nrholeritemagnet"=>"asc"));
    }


    public  function retornaRegistrosPorTipo($mesCompetencia, $listaTipoRegistro, $nrOrg, $cdBanco, $nrTipoMovimento){
        $listaTipoRegistro = explode(',',$listaTipoRegistro);
        $aux = array();
        foreach ($listaTipoRegistro as $value) {
            $aux[] = (int)$value;
        }
        return $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findBy(array("idtiporegistro"=>$aux,"nrorg"=>$nrOrg,"cdbanco"=>$cdBanco,"nrtipomoviment"=>$nrTipoMovimento));
    }


    public  function retornaRegistro($mesCompetencia, $nrOrg, $tipoRegistro, $nrInscricao, $nrVinculoM, $cdBanco, $nrTipoMovimento){
        return $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findOneBy(array("mescompetencia" => $mesCompetencia,"nrorg" => $nrOrg,"idtiporegistro" => $tipoRegistro,"cdinscestrut" => $nrInscricao,"nrvinculom" => $nrVinculoM,"cdbanco" => $cdBanco,"nrtipomoviment" => $nrTipoMovimento));
    }

    public  function retornaRegistros0Remessa($mesCompetencia, $nrOrg, $cdBanco){
        return  $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findBy(array("mescompetencia" => $mesCompetencia,"nrorg" => $nrOrg,"idtiporegistro" => Registro0HOLERITESantander::REGISTRO_0,"cdbanco" => $cdBanco),array("nrtipomoviment"=>"asc", "nreventom"=>"asc"));
    }

    /*public function procuraRegistroLista($nrInscricao, $tpRegistro, $cdBanco, $nrTipoMovimento) {
        return $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findOneBy(array("mescompetencia"=>DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia()),"nrorg"=>$this->hcmParameters->getNrorgTrabalho(),"cdbanco"=>$cdBanco,"nrtipomoviment"=>$nrTipoMovimento,"cdinscestrut"=>$nrInscricao,"idtiporegistro"=>$tpRegistro));
    }*/
    
    public function procuraRegistroLista($nrInscricao, $tpRegistro, $cdBanco, $nrTipoMovimento) {
        $itens = $this->getItems();
        foreach ($itens as $key => $item) {
            if (($item->getCdinscestrut() == $nrInscricao)
                && ($item->getIdtiporegistro() == $tpRegistro)
                && ($item->getCdbanco() == $cdBanco)
                && ($item->getNrtipomoviment() == $nrTipoMovimento)
                && ($item->getMescompetencia()->format('d/m/Y') == $this->hcmParameters->getDtCompetencia())
            ) {
                return $item;
            }
        }

        return false;
    }
    
    public function inicializaListaRegistros($mesCompetencia, $tipoRegistro, $nrOrg, $cdBanco, $nrTipoMovimento){
            $resultados = $this->retornaRegistrosPorTipo($mesCompetencia, $tipoRegistro, $nrOrg, $cdBanco, $nrTipoMovimento);
        if (is_array($resultados) === true) {
            foreach ($resultados as $resultado) {
                $this->addItem($resultado, $resultado->getNrholeritemagnet());
            }

            return true;
        } else {
            return false;
        }
    }


    public function insereRegistroLista(
        $nrOrg, $mesCompetencia, $cdBanco, $nrTipoMovimento, $idTpArquivo, $tipoRegistro, $nrInscricao, $linha, $nrSeqArquivo, $sequencia = null, $nrvinculom = null, $nrinscvinculo = null, $valor = null
    ){
        $item = new $this->fpaHoleriteMag;

        $item->setIdativo('S'); 
        $item->setNrorg($nrOrg);
        $item->setMescompetencia(DateUtil::getDataDeString($mesCompetencia,DateUtil::FORMATO_BRASILEIRO,true));
        $item->setCdbanco($cdBanco);
        $item->setNrtipomoviment($nrTipoMovimento);
        $item->setIdtparquivo($idTpArquivo);
        $item->setIdtiporegistro($tipoRegistro);
        $item->setCdinscestrut($nrInscricao);
        $item->setDslinharegistro($linha);
        $item->setNrseqarquivo($nrSeqArquivo);
        $item->setNrseqregistro($sequencia);
        $item->setNrvinculom($nrvinculom);
        $item->setCdinscvinculo($nrinscvinculo);
        $item->setValor($valor);
        $item->setNrholeritemagnet(NovoCodigo::geraCodigo($nrOrg,'FPA_HOLERITEMAGNET', 1, 22, true ));
        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $item->setDtInclusao(DateUtil::getDataAtual());
        $item->setNrorgInclusao($this->hcmParameters->getNrorgTrabalho());

        $this->addItem($item);

    }


    public function persisteRegistro(
        $nrOrg, $mesCompetencia, $cdBanco, $nrTipoMovimento, $idTpArquivo, $tipoRegistro, $nrInscricao,
        $nrSeqArquivo, $nrSeqRegistro, $linha, $nrVinculoM, $nrInscricaoVinculo, $valor
    ){
        $item = $this->retornaRegistro(
            $mesCompetencia, $nrOrg, $tipoRegistro, $nrInscricao, $nrVinculoM, $cdBanco, $nrTipoMovimento
        );

        if (is_object($item) === true) {
            $item->setNrholeritemagnet($item->getNrholeritemagnet());
            $item->setDslinharegistro($linha);
        } else {
            $item = new $this->fpaHoleriteMag();

            $item->setIdativo('S'); 
            $item->setNrorg($nrOrg);
            $item->setMescompetencia($mesCompetencia);
            $item->setCdbanco($cdBanco);
            $item->setNrtipomoviment($nrTipoMovimento);
            $item->setIdtparquivo($idTpArquivo);
            $item->setIdtiporegistro($tipoRegistro);
            $item->setCdinscestrut($nrInscricao);
            $item->setNrseqarquivo($nrSeqArquivo);
            $item->setNrseqregistro($nrSeqRegistro);
            $item->setDslinharegistro($linha);
            $item->setNrvinculom($nrVinculoM);
            $item->setCdinscvinculo($nrInscricaoVinculo);
            $item->setValor($valor);
            $item->setNrholeritemagnet(NovoCodigo::geraCodigo($nrOrg,'FPA_HOLERITEMAGNET', 1, 22, true ));
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtInclusao(DateUtil::getDataAtual());
            $item->setNrorgInclusao($this->hcmParameters->getNrorgTrabalho());
        }
        $this->entityManager->persist($item);
        $this->entityManager->flush();
    }




    public function retroceder($nrOrg, $mesCompetencia, $cdBanco, $nrTipoMovimento){

        $registros = $this->retornaRegistros($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento);
        if (count($registros) > 0) {
            foreach ($registros as $registro) {
                $this->addItem($registro);
                $this->entityManager->remove($registro);
            }
                $this->entityManager->flush();
        }
    }


    public function totalRegistrosPorTipos($mesCompetencia, $nrOrg, $tiposRegistro, $cdBanco, $nrTipoMovimento){
        $tiposRegistro = explode(",",$tiposRegistro);
        $aux = array();
        foreach($tiposRegistro as $tipo){
            $aux[] = (int)$tipo;
        }
        $totalRegistros =  $this->entityManager->getRepository(Repositories::FPA_HOLERITEMAGNET)->findBy(array("mescompetencia" => $mesCompetencia,"nrorg" => $nrOrg,"idtiporegistro" => $aux,"cdbanco" => $cdBanco,"nrtipomoviment" => $nrTipoMovimento));
        return count($totalRegistros);
    }


    public function retornaUltimoSequencialHolerite($mesCompetencia, $nrOrg, $cdBanco, $nrestruturam){
        
        /*$params = array(
            'idtiporegistro'=> Registro0HoleriteSantander::REGISTRO_0,
            'mescompetencia'=> $mesCompetencia,
            'nrorg'=> $nrOrg,
            'cdbanco'=> $cdBanco,
            'nrestruturam'=> $nrestruturam

            );
        
        $fpaholeritemagnet = $this->fpaHoleriteMag;
        
$sql = <<<SQL
SELECT MAX(m.nrseqarquivo) FROM $fpaholeritemagnet m
WHERE m.idtiporegistro = :idtiporegistro
      AND m.mescompetencia = :mescompetencia
      AND m.nrorg = :nrorg 
      AND m.cdbanco = :cdbanco 
      AND m.nrestruturam = :nrestruturam
SQL;

        $ultimoSequencial = $this->entityManager->createQuery($sql)->execute($params);
        
        if (!empty($ultimoSequencial)){
            $ultimoSequencial = end($ultimoSequencial);
            $ultimoSequencial = end($ultimoSequencial);
        }*/
        $ultimoSequencial = NovoCodigo::geraCodigo($nrOrg,'HOLERITE_NRSEQARQUIVO', 1, 6, true );
        
        return $ultimoSequencial;
    }

    public function retornaUltimoSequencialRegistro0($mesCompetencia, $nrOrg, $cdBanco){
        $params = array(
            'idtiporegistro'=> (int)Registro0HOLERITESantander::REGISTRO_0,
            'mescompetencia'=> $mesCompetencia,
            'nrorg'=> $nrOrg,
            'cdbanco'=> $cdBanco
            );
        
        $fpaholeritemagnet = $this->fpaHoleriteMag;
       
$sql = <<<SQL
SELECT MAX(m.nrseqarquivo) FROM $fpaholeritemagnet m
WHERE m.idtiporegistro = :idtiporegistro
      AND m.mescompetencia = :mescompetencia
      AND m.nrorg = :nrorg 
      AND m.cdbanco = :cdbanco 
SQL;
        $ultimoSequencial = $this->entityManager->createQuery($sql)->execute($params);
        if (!empty($ultimoSequencial)){
            $ultimoSequencial = end($ultimoSequencial)[1];
        }
        return $ultimoSequencial;
    }


    public function modificaSequenciaArquivoRemessa($mesCompetencia, $nrOrg, $cdBanco){

        $arquivosRemessa = $this->retornaRegistros0Remessa($mesCompetencia, $nrOrg, $cdBanco);

        if (!empty($arquivosRemessa)) {

            $contador = 1;
            foreach($arquivosRemessa as $arquivo) {

                $arquivo->setNrseqarquivo($contador);
                $this->entityManager->persist($arquivo);
                $this->entityManager->flush();
                $contador++;
            }
        }
    }



}
