<?php
namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteSituacaoFuncionalRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Setforms2\Shared\ORM\DateTime;
use HCM\Util\DateUtil;
use HCM\Geral\Services\ESocialService;
use HCM\FPA\Entity\FpagozoferiasEntity;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Doctrine\DBAL\Connection;
use HCM\Geral\Services\EstruturasService;

class ProgramacaoFeriasService {

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->fpaGozoFerias = new FpagozoferiasEntity($entityManager, $hcmParameters);
    }
    
    public function atualizarValoresGozo($ferias)
    {
        foreach ($ferias as &$feria) {
        
            if (is_null($feria['DTINIGOZOFERIAS'])) {
                $feria['DTINIGOZOFERIAS'] = is_null($feria['DTINIGOZO']) ? null : $feria['DTINIGOZO'];
            }
            if (is_null($feria['DTFIMGOZOFERIAS'])) {
                $feria['DTFIMGOZOFERIAS'] = is_null($feria['DTFIMGOZO']) ? null : $feria['DTFIMGOZO'];
            }
            if (is_null($feria['NRGOZOFERIAS'])) {
                $feria['NRGOZOFERIAS'] = is_null($feria['NRGOZOFERIAS2']) ? null : $feria['NRGOZOFERIAS2'];
            }
            if (!is_null($feria['FIMULTGOZO'])) {
                $dtFimUltimoGozo = $feria['FIMULTGOZO'];
                $data = DateUtil::getDataDeString($feria['FIMULTGOZO']);
                $data = DateUtil::setDay($data, $data->format("d") + 1);
                #$feria['FIMULTGOZO'] = DateUtil::truncateString($feria['FIMULTGOZO']);
                $feria['FIMULTGOZO'] = DateUtil::truncateString($data);
            }
            if ($feria['IDCONTROLFERIAS'] != 'GOZADO_PARCIAL' && is_null($feria['DIAS_RESTANTE_GOZO'])) {
                $feria['DIAS_RESTANTE_GOZO'] = 30;
            }
            
            $feria['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
        }
        
        return $ferias;
    }
    
    public function verificaDataInicio($rows) {
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        
        $validacoes = ['msg' => '', 'isValid' => true];
        
        foreach ($rows as $row) {
            $nome = $row['NMVINCULOM'];
            
            if($this->dataMenorQueCompetenciaAnterior($row['DTINIGOZOFERIAS'], $dtcompetencia)) {
                $validacoes['vinculos'][] = $nome;
                $validacoes['isValid'] = false;
            }
        }
        if(!$validacoes['isValid']) {
            $nomesVinculos = implode(', ', $validacoes['vinculos']);
            $validacoes['msg'] .=  "O Inicio Gozo de ferias do(s) Vinculo(s): $nomesVinculos devem ser preenchidos com data igual ou maior que a competencia atual!";
        }
        return $validacoes;
    }
    
    public function verificaGozoJaCadastrado($rows) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $validacoes = ['msg' => '', 'isValid' => true];
        
        foreach ($rows as $row) {
            $nome = $row['NMVINCULOM'];
            $nrvinculom = $row['NRVINCULOM'];
            $dtinigozoferias = DateUtil::truncateData(DateUtil::getDataDeString($row['DTINIGOZOFERIAS']));
            $dtfimgozoferias = DateUtil::truncateData(DateUtil::getDataDeString($row['DTFIMGOZOFERIAS']));
            
            if($this->gozoJaCadastrado($nrorg, $nrvinculom, $dtinigozoferias, $dtfimgozoferias)) {
                $validacoes['vinculos'][] = $nome;
                $validacoes['isValid'] = false;
            }
        }
        if(!$validacoes['isValid']) {
            $nomesVinculos = implode(', ', $validacoes['vinculos']);
            $validacoes['msg'] .=  "Operação bloqueada! Os seguintes vinculos: $nomesVinculos já possuem um período de gozo de férias que contempla um ou mais dias deste período!";
        }
        return $validacoes;
    }
    
    public function gozoJaCadastrado($nrorg, $nrvinculom, $dtinigozoferias, $dtfimgozoferias) {
        
        $gozoFeriasJaCadastrado = $this->fpaGozoFerias->retornaGozosFeriasEntreDatas($nrorg, $nrvinculom, $dtinigozoferias, $dtfimgozoferias);
        
        if($gozoFeriasJaCadastrado){
            return true;
        }
        return false;
    }
    
    public function dataMenorQueCompetenciaAnterior ($date, $dtcompetencia) {
        if($dtcompetencia > $date){
            return true;
        }
        return false;
    }
    
    public function saveGozoFeriasInline($rows){
        try {
            $validacoes = [];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            foreach ($rows as &$row) {
                $nrvinculom = $row['NRVINCULOM'];
                $dtinigozoferias = str_replace("/", "", $row["DTINIGOZOFERIAS"]);
                $dtfimgozoferias = str_replace("/", "", $row["DTFIMGOZOFERIAS"]);
                $dataAtual = DateUtil::getDataDeString($this->hcmParameters->getDataAtual());
                
                //Ajusta data inicial de gozo
                $dia = substr($dtinigozoferias, 0, 2);
                $mes = substr($dtinigozoferias, 2, 2) > 0 ? substr($dtinigozoferias, 2, 2) : $dataAtual->format('m');
                $ano = substr($dtinigozoferias, 4, 4) > 0 ? substr($dtinigozoferias, 4, 4) : $dataAtual->format('Y');
                $dataInicioGozo = DateUtil::getDataDeString($dia.'/'.$mes.'/'.$ano, DateUtil::FORMATO_BRASILEIRO, true);
                
                //Ajusta data fim de gozo
                $dia = substr($dtfimgozoferias, 0, 2);
                $mes = substr($dtfimgozoferias, 2, 2) > 0 ? substr($dtfimgozoferias, 2, 2) : $dataAtual->format('m');
                $ano = substr($dtfimgozoferias, 4, 4) > 0 ? substr($dtfimgozoferias, 4, 4) : $dataAtual->format('Y');
                $DataFimGozo = DateUtil::getDataDeString($dia.'/'.$mes.'/'.$ano, DateUtil::FORMATO_BRASILEIRO, true);
                $idAdiant13Sal = $row['IDADIANT13SALAR'] ? $row['IDADIANT13SALAR'] : 'N';
                $dsgozoferias = 'tela de Programacao';
                $dtRetornoFerias = DateUtil::adicionaIntervalo($DataFimGozo, 1);
                
                if(is_null($row['NRGOZOFERIAS'])){
                    $this->fpaGozoFerias->incluiGozoFerias($nrorg, $row['NRFERIAS'], $dataInicioGozo, $DataFimGozo, $row['QTDIASABONOFERI'], $dtRetornoFerias, $idAdiant13Sal, $dsgozoferias);
                    
                    $gozoFerias2 = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozoFeriasNaData(
                        $nrorg, $nrvinculom, 
                        DateUtil::getDataDeString($row["DTINIGOZOFERIAS"], DateUtil::FORMATO_BRASILEIRO), 
                        DateUtil::getDataDeString($row["DTFIMGOZOFERIAS"], DateUtil::FORMATO_BRASILEIRO)
                    );
                    
                    if(!is_null($gozoFerias2)){
                        $row['NRGOZOFERIAS'] = $gozoFerias2[0]->getNrgozoferias();
                    }
                }
                
                if($row['NRGOZOFERIAS']){
                    $gozoFerias = $this->fpaGozoFerias->retornaGozoFeriasNrGozo($nrorg, $row['NRGOZOFERIAS']);

                    if($gozoFerias){
                        $gozoFerias->setDtinigozoferias($dataInicioGozo);
                        $gozoFerias->setDtfimgozoferias($DataFimGozo);
                        $gozoFerias->setQtdiasabonoferi($row['QTDIASABONOFERI']);
                        $gozoFerias->setDtretornoferias($dtRetornoFerias);
                        $gozoFerias->setIdadiant13salar($idAdiant13Sal);
                        $gozoFerias->setDsgozoferias($dsgozoferias);
                        $gozoFerias->setDtultatu(DateUtil::getDataAtual());
                        $gozoFerias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $gozoFerias->setCdoperultatu($this->hcmParameters->getCdoperador());
                
                        $this->entityManager->persist($gozoFerias);
                        $this->entityManager->flush($gozoFerias);
                    }
                    
                }else{
                    $this->fpaGozoFerias->incluiGozoFerias($nrorg, $row['NRFERIAS'], $dataInicioGozo, $DataFimGozo, $row['QTDIASABONOFERI'], $dtRetornoFerias, $idAdiant13Sal, $dsgozoferias);
                }
            }
        } catch (Exception $e) {
            throw $e;    
        }
    }
} 