<?php
/**
 * Responsável pela tela de relatórios gerais do HCM
 * Rafael Pickbrenner
 * 28/10/2016
 * 
 * 19/06/2018 Funcionalidade para download de arquivo/relatório. Emissão em outros formatos. Escolha da extensão do arquivo gerado.
 */

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Geral\Services\VinculoServices;
use HCM\Geral\Services\RelatoriosHcmService;
use HCM\Geral\Controllers\EnviarMensagemController;

class RelatoriosHcmController extends \Zeedhi\Framework\Controller\Report {

    const FORMAT_PDF = '.pdf';

    /** @var array  */
    protected $reportMapping = array ();
    protected $hcmParameters;
    protected $enviarMensagemController;
    protected $relatoriosHcmService;
    protected $vinculoServices;

    public function __construct(ReportService $reportService, HCMParameters $hcmParameters, RelatoriosHcmService $relatoriosHcmService, VinculoServices $vinculoServices, EnviarMensagemController $enviarMensagemController){
        parent::__construct($reportService);

        $this->hcmParameters            = $hcmParameters;
        $this->enviarMensagemController = $enviarMensagemController;        
        $this->relatoriosHcmService     = $relatoriosHcmService;
        $this->vinculoServices          = $vinculoServices;
    }
    
    /*
     * Abre relatórios Birt. Usada na tela 'RelatoriosHcm'. Chamada no 'RelatoriosHcmService.js'.
     * Recebe da tela os parâmetros específicos do relatório e o 'reportMapping' (montado no service)
     * e incluí os parâmetros padrões. A inclusão destes parâmetros na backend oferece mais segurança.
     */
    public function setRelatoriosHcm(DTO\Request\DataSet $request, DTO\Response $response){
        error_reporting(E_ERROR | E_PARSE);
        //Adquiri a row com o 'campo mapping' contendo o reportMapping
        $row = $request->getDataSet()->getRows();
        $row = $row[0];                //Ajuste de índice/formato

        if(!isset($row['isLote'])){ //Requisições do RCB não possuem array de 'row', criando para manter a compatibilidade abaixo
            $mapping     = $row['mapping'];     //Extraindo apenas o reportMapping
            $nmReport    = key($mapping);       //Nome do relatório (índice para o array do reportMapping)
            $hasDownload = isset($row['download']);     //Flag para indicar que a rotina de downlaod deve ser acionada
            
            //Parâmetros padrão - row
            $this->setParametrosPadrao($row);
            
            //Parâmetros padrão - reportMapping
            $this->setReportMappingDefault($nmReport, $mapping);
            // Ajusta o reportMapping para o relatório em questão
            $this->reportMapping = $mapping;
            //Refazendo a requisição com os parâmetros padrões
    	    $newRequest = new \Zeedhi\Framework\DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    
            //TODO Verificar campo de array se deve ou não emitir o relatório (ver exemplo completo no requisicao_seguro_desemprego, /hcm/mobile/assets/js/utilities/RelatoriosHcmConfig.js)
            parent::createRemoteReport($newRequest, $response);         //Abre o relatório
            
            //Envia a requisição para download do arquivo se necessário
            if($hasDownload){
                $caminho = $this->getFileDownloaded($row, $mapping, $newRequest); //Adquiri o caminho do arquivo já baixado em uma pasta temporária
                $row = array('NMARQUIVO' => $caminho);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $row));
            }
        } else {
            $lote = $request->getDataSet()->getRows();
            $lote = $lote[0];                //Ajuste de índice/formato
            
            unset($lote['isLote']);         //Para não participar do percorre relatórios abaixo
            unset($lote['__report_name']);

            foreach($lote as $index) { //percorre todos os relatórios do lote para a emissão destes
                foreach($index as $aux=>$row){  
                    $row = new \Zeedhi\Framework\DTO\Row($row);
                    $mapping     = $row['mapping'];     //Extraindo apenas o reportMapping
                    $nmReport    = key($mapping);       //Nome do relatório (índice para o array do reportMapping)
                    $hasDownload = isset($row['download']);     //Flag para indicar que a rotina de downlaod deve ser acionada
                    
                    //$row['__report_name'] = $aux; //'Restaurando' o nome do relatório para as funções do Zeedhi
                    $row['__report_name'] = $row['fileName']; //'Restaurando' o nome do relatório para as funções do Zeedhi
                    unset($row['fileName']);
                    
                    //Parâmetros padrão - row
                    $this->setParametrosPadrao($row);
                    //Parâmetros padrão - reportMapping
                    $this->setReportMappingDefault($nmReport, $mapping);
                    // Ajusta o reportMapping para o relatório em questão
                    $this->reportMapping = $mapping;
        
                    //Refazendo a requisição com os parâmetros padrões
            	    $newRequest = new \Zeedhi\Framework\DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
                    //TODO Verificar campo de array se deve ou não emitir o relatório (ver exemplo completo no requisicao_seguro_desemprego, /hcm/mobile/assets/js/utilities/RelatoriosHcmConfig.js)
                    parent::createRemoteReport($newRequest, $response);         //Abre o relatório
                    
                    //Envia a requisição para download do arquivo se necessário
                    if($hasDownload){
                        $caminho = $this->getFileDownloaded($row, $mapping, $newRequest); //Adquiri o caminho do arquivo já baixado em uma pasta temporária
                        $row = array('NMARQUIVO' => $caminho);
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $row));
                    }
                }    
            }
        }
    }
    
    public function setRelatoriosHcmBackend($params, $nmReport, $request = null, $response = null){ // função criada para buscar relatorio via php, caminho do relatorio retornado no response
        $mapping     = $this->createMapping($params, $nmReport);
        $hasDownload = isset($params['download']); 
        $row = new \Zeedhi\Framework\DTO\Row($params);
        $this->setParametrosPadrao($row);
        $this->setReportMappingDefault($nmReport, $mapping);
        $this->reportMapping = $mapping;
        
        $row["__report_name"] = $nmReport;

        if ($request) {
	        $newRequest = new \Zeedhi\Framework\DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        } else {
            $newRequest = new \Zeedhi\Framework\DTO\Request\Row($row, "", "", "");
        }
        
        if ($response)
            parent::createRemoteReport($newRequest, $response);
        
        if($hasDownload){
            $row['mapping'] = $mapping;
            $caminho = $this->getFileDownloaded($row, $mapping, $newRequest); //Adquiri o caminho do arquivo já baixado em uma pasta temporária
            $params = array('NMARQUIVO' => $caminho);
            if($response) 
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $params));
        }
        
        return $this->getUrlReport($newRequest);
    }
    
    private function createMapping($params, $nmReport) {
        $aux = array();
        foreach ($params as $key=>$parametros) {            
            if($key != 'download')
                $aux[$key] = $key;  
        }

        $mapping = array(
                $nmReport=>
                  array(
                    "strategy" => "BIRT",
                    "parameterMapping"=> $aux
                )
        );
        return $mapping;
    }
    
    private function setParametrosPadrao(&$row) {
        $row["NRORG"]          = $this->hcmParameters->getNrorgTrabalho();
        $row["NRORG_PADRAO"]   = $this->hcmParameters->getNrorgPadrao();
        $row["CDOPERADOR"]     = $this->hcmParameters->getCdoperador();
        $row["NMOPERADOR"]     = $this->hcmParameters->getNmoperador();
        $row["NROCORRECAL"]    = $this->hcmParameters->getOcorrenciaCalculo();
        $row["OCORRECAL"]      = $this->hcmParameters->getOcorrenciaCalculo();
        $row["MODALIDADECAL"]  = $this->hcmParameters->getNrModalidadeCalculo();
        $row["NRTPMODALIDCAL"] = $this->hcmParameters->getNrModalidadeCalculo();
        //$row["CONFIDENCIAL"] = 'S';
        
        $nrestruttrabOper = FPA::getNrestrutlegalOper();
        
        if($nrestruttrabOper){
            $row["NRESTRUTLEGAL"] = !$row["NRESTRUTLEGAL"] ? $nrestruttrabOper : $row["NRESTRUTLEGAL"];
            $row["NRESTRUTLEGAL_LIST"] = !$row["NRESTRUTLEGAL_LIST"] ? $nrestruttrabOper : $row["NRESTRUTLEGAL_LIST"];
        }
    }

    private function setReportMappingDefault($nmReport, &$mapping) {
        $mapping[$nmReport]['parameterMapping']['NRORG'         ] = 'P_NRORG';
        $mapping[$nmReport]['parameterMapping']['NRORG_PADRAO'  ] = 'P_NRORG_PADRAO';
        $mapping[$nmReport]['parameterMapping']['NROCORRECAL'   ] = 'P_NROCORRECAL';
        $mapping[$nmReport]['parameterMapping']['OCORRECAL'     ] = 'P_OCORRECAL';
        $mapping[$nmReport]['parameterMapping']['MODALIDADECAL' ] = 'P_MODALIDADECAL';
        $mapping[$nmReport]['parameterMapping']['NRTPMODALIDCAL'] = 'P_NRTPMODALIDCAL';
        $mapping[$nmReport]['parameterMapping']['CDOPERADOR'    ] = 'CDOPERADOR';
        $mapping[$nmReport]['parameterMapping']['NMOPERADOR'    ] = 'NMOPERADOR';
        //$mapping[$nmReport]['parameterMapping']['CONFIDENCIAL'    ] = 'P_CONFIDENCIAL';
    }
    
    private function getFileDownloaded($row, $mapping, $request) {
        //Dados do arquivo/relatório a ser baixado extraído do array do frontend
        $nmReport   = key($row['mapping']);
        $fileName   = isset($row['download']['fileName'])   ? $row['download']['fileName']   : $nmReport;
        $isEval     = isset($row['download']['isEval'])     ? $row['download']['isEval']     : false; //Usado para montar nomes de arquivos dinâmicos
        $downFormat = isset($row['download']['format'])     ? $row['download']['format']     : 'pdf';
        $extension  = isset($row['download']['extension'])  ? $row['download']['extension']  : $downFormat;
        $strFind    = isset($row['download']['strFind'])    ? $row['download']['strFind']    : null;
        $strReplace = isset($row['download']['strReplace']) ? $row['download']['strReplace'] : null;
        
        if($isEval){ //Se for um nome dinâmico de arquivo este o monta aqui
            $fileName = "\$fileName = ".$fileName.";";
            eval($fileName);
            //Caminho completo do relatório a ser baixado, com nome de arquivo dinâmeico, sem o número de org e operador concatenado
            $fileNamePath = $this->hcmParameters->getTempDirectory().'/'.$fileName.'.'.$extension;
        } else {
            //Caminho completo do relatório a ser baixado, com nome de arquivo e número de org e operador concatenado
            $fileNamePath = $this->hcmParameters->getTempDirectory().'/'.$fileName.'_'.$row['NRORG'].'_'.$row['CDOPERADOR'].'.'.$extension;
        }
        
        //Ajuste do formato do relatório para o download apenas
        $mapping[key($mapping)]['parameterMapping']['formato'] = "__format"; //Sobreescreverá o parâmetro de formato default do relatório pelo valor da row
        $row['formato'] = $downFormat;
        
        //Re-ajusta o reportMapping para o relatório em questão (os parâmetros padrões já foram setados)
        $this->reportMapping = $mapping;
        //Refazendo a requisição com os parâmetros padrões
	    //$newRequest = new \Zeedhi\Framework\DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());

        //Aquisição e adequação da url do relatório a ser baixado
        $reportURL = parent::processReport($request);
        $reportURL = str_replace(' ', '%20', $reportURL); //Ajusta os espaços em branco
        
        //Adquiri o conteúdo do arquivo e, se necessário, faz o ajuste (replace) de strings
        $fileContents = file_get_contents($reportURL);
        if(!is_null($strFind)) {
            $fileContents = str_replace($strFind, $strReplace, $fileContents);
        }
        
        //Salva o arquivo recém adquirido
        file_put_contents($fileNamePath, $fileContents);
        
        return $fileNamePath;
    }

    private function getUrlReport($row) {
        $responseReport = new DTO\Response();
        parent::createRemoteReport($row, $responseReport);
        $messages = $responseReport->getMessages();        
        return end($messages)->getMessage();
    }

    public function sendReportEmail(DTO\Request\Row $row, DTO\Response $response) {
        set_time_limit(0);
        

        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $competencia    = substr(str_replace('/', '', $mesCompetencia), 2);
        $nrOrg          = $this->hcmParameters->getNrorgTrabalho();  

        $dataset  = $row->getRow();
        $mapping  = $dataset['mapping'];
        $nmReport = key($mapping);

        // substitui os mnemonicos encontrados na mensagem
        $dataset["MENSAGEM"]["DSMENSAGEM"] = $this->factoryMnemonico($dataset, $nmReport);

        $ordem          = $dataset['MENSAGEM']['DSORDEM'];
        $dsInicio       = $dataset['MENSAGEM']['DSINICIO'];
        $dsFim          = $dataset['MENSAGEM']['DSFIM'];
        $mensagem       = $dataset['MENSAGEM'];        

        // remove as posições abaixo por serem descenessárias a partir daqui
        unset($row->getRow()["MENSAGEM"]);
        unset($row->getRow()["FORMULARIO"]);

        // parâmetros padrão
        $row->getRow()["NRORG"]          = $this->hcmParameters->getNrorgTrabalho();
        $row->getRow()["NRORG_PADRAO"]   = $this->hcmParameters->getNrorgPadrao();
        $row->getRow()["CDOPERADOR"]     = $this->hcmParameters->getCdoperador();
        $row->getRow()["NMOPERADOR"]     = urlencode(htmlentities($this->hcmParameters->getNmoperador()));
        $row->getRow()["NROCORRECAL"]    = $this->hcmParameters->getOcorrenciaCalculo();
        $row->getRow()["NRTPMODALIDCAL"] = $this->hcmParameters->getNrModalidadeCalculo();
        $row->getRow()["__report_name"]  = $nmReport;

        // parâmetros padrão - reportMapping
        $this->setReportMappingDefault($nmReport, $mapping);
        // ajusta o reportMapping para o relatório em questão
        $this->reportMapping = $mapping;                

        // se tipo de vínculo estiver setado, busca pelo tipo de vinculo
        if (isset($dataset["NRTIPOVINCULOM"]) && !is_null($dataset["NRTIPOVINCULOM"]))
            $vinculos = $this->vinculoServices->retornaVinculosCompetencia($nrOrg, $mesCompetencia, false, $dsInicio, $dsFim, $ordem, true, null, true, $dataset["NRTIPOVINCULOM"]);
        else 
            $vinculos = $this->vinculoServices->retornaVinculosCompetencia($nrOrg, $mesCompetencia, false, $dsInicio, $dsFim, $ordem, true);

        $returnDsEmailOper = null;
        $returnAmbos       = null;
        // processa os vinculos retornados
        foreach ($vinculos as $vinculo) {            
            $row->getRow()['NRVINCULOM'] = $vinculo->NRVINCULOM;    
                                    
            // verifica regras para emissão do relatório
            $rulesReports = $this->rulesReports(array(
                "P_NMREPORT" => $nmReport,
                "P_VINCULO"  => $vinculo,
                "P_NRORG"    => $row->getRow()["NRORG"]
            ));
            if (!$rulesReports) {
                 continue;
            }

            // buscar url do relatorio e escreve o arquivo com conteúdo do mesmo
            $reportUrl = $this->getUrlReport($row);
            $fileName  = $vinculo->NRVINCULOM.'_'.$nmReport.'_'.$competencia.self::FORMAT_PDF;
            $filePath  = $this->gravarArquivoRelatorio($fileName, $reportUrl);

            if (file_exists($filePath)) {                
                $return = $this->enviarMensagemController->enviarEmailDemonstrativo($mensagem, $nrOrg, $mesCompetencia, $vinculo->NRVINCULOM, $vinculo->historico->getNrestrutlegal(), $filePath);
                if (is_bool($return)) {
                    unlink($filePath);
                    $returnVinculo[] = $vinculo->NMVINCULOM;
                } else {
                    if($return == 'DSEMAILOPER'){
                        if(!is_null($returnDsEmailOper)){
                            $returnDsEmailOper .= ' ';
                        }
                        $returnDsEmailOper.= $vinculo->NMVINCULOM;
                    } else if ($return == 'AMBOS'){
                        if(!is_null($returnAmbos)){
                            $returnAmbos .= ' ';
                        }
                        $returnAmbos .= $vinculo->NMVINCULOM;
                    }
                }
            } else {
                throw new \Exception(Messages::HCM01000_FPA_ERROR_SAVE_REPORT, 1);                
            }
        }
        
        $message = $this->returnMessage($returnDsEmailOper, $returnAmbos);
        if(!is_null($message)){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
        }
    }

    private function gravarArquivoRelatorio($fileName, $reportUrl) {
        $fileContent = \HCM\Util\HCM::loadReport($reportUrl);
        $filePath    = \HCM\Util\HCM::gravarArquivo($fileName, $fileContent);
        return $filePath;
    }

    private function rulesReports($parameters) {
        return $this->relatoriosHcmService->rulesReports($parameters);
    }

    private function returnMessage($returnDsEmailOper, $returnAmbos) {
        $message = null;
        if(!is_null($returnDsEmailOper)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_EMAIL_NOTFOUND, $returnDsEmailOper);
        }

        if(!is_null($returnAmbos)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_OPERADOR_NOTFOUND. ' ou forma de comunicação', $returnAmbos);
        }

        return $message;
    }

    private function factoryMnemonico($dataset, $nmReport) {
        switch ($nmReport) {
            case RelatoriosHcmService::RPT_PONTO_DEMONSTRATIVO:
                $replaceMnemonico = array(
                    "[PERIODO_APURACAO]" => str_replace('(FECHADO)', '', $dataset["FORMULARIO"]["DSPERIODOAPURACAO"])
                );
                return $this->relatoriosHcmService->replaceMnemonicos($dataset["MENSAGEM"]["DSMENSAGEM"], $replaceMnemonico);
                break;
            
            default:
                break;
        }
    }
}