<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\MovimentacaoRepository;
use HCM\VAB\Services\ValeTransporteService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use Teknisa\Model\Geral\Entities\Tipoestrutura;
use Teknisa\Model\Gestaopessoas\Entities\GpeTipotransfer;
use Zeedhi\Framework\DTO\Response;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;

class MovimentacaoService {

    const ALTERA_VINCULO = 'ALTERA';

    private $dataInicialUltimaMovimentacaoLegalVinculo = [];
    
    public function __construct(
        EntityManager $entityManager, 
        HCMParameters $hcmParameters, 
        VinculoServices $vinculoServices, 
        MovimentacaoRepository $movimentacaoRepository,
        ValeTransporteService $valeServices, 
        ExportacaoExcelService $exportacaoExcelService,
        Kernel $httpKernel
    ) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->vinculoServices = $vinculoServices;
        $this->movimentacaoRepository = $movimentacaoRepository;
        $this->valeTransporteServices = $valeServices;
        
        $this->alteSituFuncEntity = new GpealtesitufuncEntity($entityManager, $hcmParameters);
        $this->gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
    }
    
    public function getMovimentacaoCrud ($nrorg, $nrorg_padrao, $dtmescompetenc, $conditions, $orderBy)
    {
        return $this->movimentacaoRepository->getMovimentacaoCrud($nrorg, $nrorg_padrao, $dtmescompetenc, $conditions, $orderBy);
    }
    
    public function beforeSave ($nrorg, $nrvinculom)
    {
        $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom);
    }
    
    public function retornaHierarquiaValida($nrorg, $nrtpestrut, $nrtipoestrutrel = null){
        return $this->movimentacaoRepository->retornaHierarquiaValida($nrorg, $nrtpestrut, $nrtipoestrutrel);
    }
    
    
    public function retornaMovimentacaoLegal($nrorg, $nrtipoestrutura, $nrvinculom)
    {
        return $this->movimentacaoRepository->getMovimentacao($nrorg, $nrtipoestrutura, $nrvinculom);
    }
    
    public function retornaMovimentacaoLegalEntrada($nrorg, $nrtipoestrutura, $nrvinculom)
    {
        return $this->movimentacaoRepository->getMovimentacaoEntrada($nrorg, $nrtipoestrutura, $nrvinculom);
    }
    
    public function validaHierarquiaGerenLegal($nrorg, $nrestruturalegal, $nrestruturam, $nrtipotransfer, $nrtipoestrutura, $nrtipoestruturaGerenOrg, $nrtipoestruturaLegalOrg)
    {
        if($nrtipotransfer == 2 && $nrtipoestrutura == $nrtipoestruturaGerenOrg)
        {
            $gerenRelacionadaHierarqLegal = $this->movimentacaoRepository->retornaHierarquiaValidaPorEstrutura($nrorg, $nrestruturalegal->getNrestruturam(), $nrestruturam, $nrtipoestruturaLegalOrg, $nrtipoestruturaGerenOrg);
            /*Se a estrutura gerencial NÃO estiver relacionada a legal, deve ser bloqueado o cadastro*/
            if(!$gerenRelacionadaHierarqLegal)
            {
                return false;
            }
            return true;
        }
        return true;
    }

    public function afterSave ($nrorg, $nrvinculom, $nrestruturam, $nrtipoestrutura, $dtinimoviment, $dtfimmoviment, $iutilizamatric = null, $isnew = null, $nrtipotransfer = null)
    {
        $matricula = [];
        
        if($dtfimmoviment) $dtfimmoviment = DateUtil::getDataDeString(substr($dtfimmoviment, 0, 10), DateUtil::FORMATO_BRASILEIRO);
        
        $proxmovimentacao = $this->movimentacaoRepository->retornaProximaMovimentacaoPorTipoestrutura($nrorg, $nrvinculom, $nrtipoestrutura, $dtinimoviment, $nrtipotransfer);
        if ($proxmovimentacao) {
            $dtiniproxmoviment = DateUtil::getDataDeString(substr($proxmovimentacao->getDtinimoviment(), 0, 10), DateUtil::FORMATO_BRASILEIRO);
            $dtiniproxmoviment->sub(new \DateInterval('P1D'));
            $dtfimmoviment = $dtiniproxmoviment;
        }
        
        $dtinimoviment = DateUtil::getDataDeString(substr($dtinimoviment, 0, 10), DateUtil::FORMATO_BRASILEIRO);
        $ultDiaCompetenc = DateUtil::getDataDeString(substr($this->hcmParameters->getUltimoDiaMes(), 0, 10), DateUtil::FORMATO_BRASILEIRO);
        if (
               // Se a DT. inicial da movimentação é menor ou igual ao último dia da competência.
               $dtinimoviment->format('Y/m/d') <= $ultDiaCompetenc->format('Y/m/d') 
               // Se não houver DT. final da movimentação, ou cajo haja, seja maior ou igual ao último dia da competência.
            && (
                    !$dtfimmoviment
                    || $dtfimmoviment == ''
                    || $ultDiaCompetenc->format('Y/m/d') <= $dtfimmoviment->format('Y/m/d')  
               )
        ) {
            $this->atualizaEstruturaHistoricoVinculo($nrorg, $nrvinculom, $nrtipoestrutura, $nrestruturam, $nrtipotransfer);
        }
        
        if($nrtipoestrutura == $this->hcmParameters->getTipoEstruturaLegalOrg()){
            $matricula = $this->alteraMatriculaEsocial($nrestruturam, $nrorg, $nrvinculom, $iutilizamatric, $isnew);
        }
        
        return $matricula;
    }

    public function verificaTipoTransferencia($nrtipotransfer, $nrtipoestrutura){
        try {
            if ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL) {
                if ($nrtipoestrutura != $this->hcmParameters->getTipoEstruturaLegalOrg() && $nrtipoestrutura != Tipoestrutura::SINDICATO) {
                    throw new \Exception(Messages::HCM00000_GPE_MOVIMENTACAO_LEGAL_TIPOESTRUT_ERROR, Messages::ERROR_MESSAGE_CODE);
                }
            } elseif ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN) {
                if (($nrtipoestrutura == $this->hcmParameters->getTipoEstruturaLegalOrg() && $this->hcmParameters->getTipoEstruturaLegalOrg() != $this->hcmParameters->getTipoEstruturaGerencialOrg()) || $nrtipoestrutura == Tipoestrutura::SINDICATO) {
                    throw new \Exception(Messages::HCM00000_GPE_MOVIMENTACAO_GEREN_TIPOESTRUT_ERROR, Messages::ERROR_MESSAGE_CODE);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function alteraMatriculaEsocial($nrestruturam, $nrorg, $nrvinculom, $iutilizamatric, $isnew){
        try {
            $dtmescompetenc = $this->hcmParameters->getDtcompetencia();
            $matricula['NOVO'] = null;
            
            if($isnew === true){
                if($iutilizamatric == 'S'){
                    $matricula = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->retonaUltimaMatriculaEsoIncrementada($nrorg, $nrestruturam, $dtmescompetenc);
                }
                if ($this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc)->getCdmatriculaesocial() == null) {
                    $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);
                    
                    $this->entityManager->beginTransaction();
                    
                    $vinculo->setCdmatriculaesocial($matricula['NOVO']);
                    
                    $this->entityManager->persist($vinculo);
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                }
            }else{
                $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);
                $matricula['NOVO'] = $vinculo->getCdmatriculaesocial();
            }
            return $matricula;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /**
     * Valida se já existe outra movimentacao para o vínculo com a mesma data
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $dtinimoviment
     * @param $dtfimmoviment
     * @param $nrtipoestrutura
     * @param $nrmovimentacao
     * @throws \Exception
     */
    public function validaExisteDataMovimentacao($nrorg, $nrvinculom, $dtinimoviment, $dtfimmoviment, $nrtipoestrutura, $nrmovimentacao, $nrtipotransfer) {
        try {
            $gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
            $qb = $this->entityManager->createQueryBuilder();
            // Em caso de alteração desconsidera o próprio registro
            if ($nrmovimentacao == '') {
                $andNrMov = '1 = 1';
            } else {
                $andNrMov = 'm.nrmovimentacao <> '.$nrmovimentacao;
            }

            // Permite duas movimentações para a mesma data e mesmo tipo de estrutura sendo uma delas com data final ou data final diferente da nova
            if ($dtfimmoviment == '') {
                $andDtFim = 'm.dtfimmoviment is null';
            } else {
                $andDtFim = "m.dtfimmoviment is not null and m.dtfimmoviment = '".$dtfimmoviment."'";
            }

            // Verifica se há um ou mais registros com a mesma data de inicio e tipo de estrutura
            $query = $qb
                ->select('m')
                ->from($gpeMovimentacao, 'm')
                ->where(
                    $qb->expr()->eq('m.nrorg', $nrorg),
                    $qb->expr()->eq('m.nrvinculom', $nrvinculom),
                    $qb->expr()->eq('m.nrtipoestrutura', $nrtipoestrutura),
                    $qb->expr()->eq('m.nrtipotransfer', $nrtipotransfer)
                )
                ->andWhere("m.dtinimoviment = '".$dtinimoviment."'")
                ->andWhere($andNrMov)
                ->andWhere($andDtFim)
                ->getQuery();

            $movimentacao = $query->execute();

            if (!empty($movimentacao)) {
                throw new \Exception(Messages::HCM00000_GER_MOVIMENTACAO_DATA_ALREADY_EXIST_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza número da estrutura da movimentação no histórico atual do vínculo
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $nrtipoestrutura
     * @throws \Exception
     */
    public function atualizaEstruturaHistoricoVinculo($nrorg, $nrvinculom, $nrtipoestrutura, $nrestruturam = null, $nrtipotransfer = null, $delete = false) {
        try {
            if (is_null($nrestruturam)) {
                $gpemovimentacao = $this->movimentacaoRepository->getMovimentacao($nrorg, $nrtipoestrutura, $nrvinculom, $nrtipotransfer);
                if(is_object($gpemovimentacao)){
                    $nrestruturam = $gpemovimentacao->getNrestruturam();
                } else {
                    $nrestruturam = null;
                }
            }

            $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom);
            $historicoAnterior = $this->vinculoServices->retornaHistoricoAnteriorUltimoHistorico($nrorg, $nrvinculom, $gpevinculoh->getNrvinculoh(), $this->hcmParameters->getDtCompetencia());
            $matricula = [];
            
            if($this->hcmParameters->getTipoEstruturaLegalOrg() == $this->hcmParameters->getTipoEstruturaGerencialOrg()){
                if($nrtipoestrutura != Tipoestrutura::SINDICATO){    
                    if($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL){
                        $gpevinculoh->setNrestrutlegal($nrestruturam);
                    }else if($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN){
                        $gpevinculoh->setNrestrutgeren($nrestruturam);
                    }
                }else{
                    $gpevinculoh->setNrestrutsind($nrestruturam);
                }
            }else{
                switch ($nrtipoestrutura){
                    case $this->hcmParameters->getTipoEstruturaLegalOrg():
                        $gpevinculoh->setNrestrutlegal($nrestruturam);
                        break;
                    case $this->hcmParameters->getTipoEstruturaGerencialOrg():
                        $gpevinculoh->setNrestrutgeren($nrestruturam);
                        break;
                    case Tipoestrutura::SINDICATO:
                        $gpevinculoh->setNrestrutsind($nrestruturam);
                        break;
                }
            }
            
            
            if($delete === true && $nrtipoestrutura == $this->hcmParameters->getTipoEstruturaLegalOrg() && $nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL){
                if($historicoAnterior){
                    if($historicoAnterior['CDMATRICULAESOCIAL']){
                        $gpevinculoh->setCdmatriculaesocial($historicoAnterior['CDMATRICULAESOCIAL']);
                    }else{
                        $gpevinculoh->setCdmatriculaesocial(null);
                    }
                }
            }
            
            
            if($delete === true) {
                if($this->hcmParameters->getTipoEstruturaLegalOrg() == $this->hcmParameters->getTipoEstruturaGerencialOrg()) {
                    if($nrtipoestrutura != Tipoestrutura::SINDICATO) {
                        if($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL) {
                            $nrestrutlegalAnterior = $historicoAnterior['NRESTRUTLEGAL'];
                            if ($nrestrutlegalAnterior !== $nrestruturam) {
                                $gpevinculoh->setNrestrutlegal($nrestrutlegalAnterior);
                            } else {
                                $gpevinculoh->setNrestrutlegal(null);
                            }
                        } else if ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN) {
                            $nrestrutgerenAnterior = $historicoAnterior['NRESTRUTGEREN'];
                            if ($nrestrutgerenAnterior !== $nrestruturam) {
                                $gpevinculoh->setNrestrutgeren($nrestrutgerenAnterior);
                            } else {
                                $gpevinculoh->setNrestrutgeren(null);
                            }
                        }
                    } else {
                        $nrestrutsindAnterior = $historicoAnterior['NRESTRUTSIND'];
                        if ($nrestrutsindAnterior !== $nrestruturam) {
                            $gpevinculoh->setNrestrutsind($nrestrutsindAnterior);
                        } else {
                            $gpevinculoh->setNrestrutsind(null);
                        }
                    }
                } else {
                    switch ($nrtipoestrutura) {
                        case $this->hcmParameters->getTipoEstruturaLegalOrg():
                            $nrestrutlegalAnterior = $historicoAnterior['NRESTRUTLEGAL'];
                            if ($nrestrutlegalAnterior !== $nrestruturam) {
                                $gpevinculoh->setNrestrutlegal($nrestrutlegalAnterior);
                            } else {
                                $gpevinculoh->setNrestrutlegal(null);
                            }
                            break;
                        case $this->hcmParameters->getTipoEstruturaGerencialOrg():
                            $nrestrutgerenAnterior = $historicoAnterior['NRESTRUTGEREN'];
                            if ($nrestrutgerenAnterior !== $nrestruturam) {
                                $gpevinculoh->setNrestrutgeren($nrestrutgerenAnterior);
                            } else {
                                $gpevinculoh->setNrestrutgeren(null);
                            }
                            break;
                        case Tipoestrutura::SINDICATO:
                            $nrestrutsindAnterior = $historicoAnterior['NRESTRUTSIND'];
                            if ($nrestrutsindAnterior !== $nrestruturam) {
                                $gpevinculoh->setNrestrutsind($nrestrutsindAnterior);
                            } else {
                                $gpevinculoh->setNrestrutsind(null);
                            }
                            break;
                    }
                }
            }
            
            
            $gpevinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gpevinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculoh->setDtultatu(DateUtil::getDataAtual());

            $this->entityManager->persist($gpevinculoh);
            $this->entityManager->flush($gpevinculoh);
            
            return array('ANTIGA' => $matricula);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Função para manutenção de movimento de vínculos em uma estrutura, gerando uma nova movimentação para
     * todos os vínculos de uma estrutura selecionada (filtro por tipo de estrutura, estrutura e multsearch de vínculo).
     * metodo no metadados
     */
    public function manutencaoMovimentacaoVinculo($params, Response $response) {
        try {
            $nrtipotransfer = $params['NRTIPOTRANSFER'];
            $nrestruturamori = $params['NRESTRUTURAMORIG'];
            $nrtipoestruturaori = $params['NRTIPOESTRUTURAORIG'];
            $vinculos = $params['NRVINCULOM'];
            $nrtpmovtransfm = $params['NRTPMOVTRANSFM'];
            $nrmotivotransf = $params['NRMOTIVOTRANSF'];
            $dtinimoviment = $params['DTINIMOVIMENT'];
            $dtfimmoviment = $params['DTFIMMOVIMENT'];
            $idatualizaacerto = $params['IDATUALIZAACERTO'];
            $nrtipoestrutura = $params['NRTIPOESTRUTURA'];
            $nrestruturam = $params['NRESTRUTURAM'];
            $dsobservacao = $params['DSOBSERVACAO'];
            $nrtipotransferdestino = $params['NRTIPOTRANSFERDEST']; 
            $inclusaoMovimentacaoSaida = $params['INCLUSAOAUTOMATICAMOVSAIDA'];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $ultimodiames = $this->hcmParameters->getUltimoDiaMes();
            

            if (empty($vinculos)) {
                $listavinculos = 'A';
            } else {
                $listavinculos = implode(',',$vinculos);
            }
            
            $movimentacoes = $this->movimentacaoRepository->getMovimentacoesPorEstrutura($nrorg, $ultimodiames, $nrestruturamori, $nrtipoestruturaori, $listavinculos, $nrtipotransfer);
            if (empty($movimentacoes)) {
                $response->addMessage(new Response\Message(Messages::HCM00000_GER_MOVIMENTACAO_NOT_FOUND), Response\Message::TYPE_ERROR);
            }
            
            $inclusao = false;
            $msg_vinculos = "";

            if ($inclusaoMovimentacaoSaida == 'S') {
                $this->setDataInicialUltimaMovimentacaoLegalParaCadaVinculo($movimentacoes);
            }

            foreach ($movimentacoes as $movimentacao) {
                $nrvinculom = $movimentacao->getNrvinculom();
                $nrestruturalegal = $this->retornaMovimentacaoLegalEntrada($nrorg, $this->hcmParameters->getTipoEstruturaLegalOrg(), $nrvinculom);     

                if($nrestruturalegal){
                    $valida = $this->validaHierarquiaGerenLegal($nrorg, $nrestruturalegal, $nrestruturam, $nrtipotransfer, $nrtipoestrutura, $this->hcmParameters->getTipoEstruturaGerencialOrg(), $this->hcmParameters->getTipoEstruturaLegalOrg());
                    if($valida == false){
                        $msg_vinculos .= "<br>-Não há Hierarquia relacionando a Estrutura Gerencial escolhida com a Estrutura Legal atual do vínculo! <br>";
                        continue;
                    }
                }
                
                if ($inclusaoMovimentacaoSaida == 'S' && $movimentacao->getDtinimoviment() == $this->dataInicialUltimaMovimentacaoLegalVinculo[$nrvinculom]) {
                    $this->setMovimentacaoSaida($movimentacao, $params);
                }
                
                $ultimoHistoricoVinculo = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom);
                if (!$ultimoHistoricoVinculo) {
                    continue;
                }
                
                //Verifica Afastamento
                $afastado = $this->alteSituFuncEntity->verificaVinculoAfastado($nrorg, $nrvinculom, DateUtil::getDataDeString($dtinimoviment));
                
                if($afastado){
                    $msg_vinculos .= " O vínculo $nrvinculom - está com situação funcional de afastamento <br>";
                    continue;
                }
                
                $movimentacaoDtIni = DateUtil::truncateString($movimentacao->getDtinimoviment());
                $dtinimovimentacao = DateUtil::getDataDeString($dtinimoviment, DateUtil::FORMATO_BRASILEIRO, true);
                $comparadatas = DateUtil::comparaDatas($movimentacao->getDtinimoviment(), $dtinimovimentacao);

                if(!($movimentacaoDtIni == $dtinimoviment && $movimentacao->getNrestruturam() == $nrestruturam) && ($comparadatas || $movimentacaoDtIni == $dtinimoviment)){
                    // Verifica se o tipo a ser inserido é diferente do tipo filtrado
                    if($nrtipotransfer != $nrtipotransferdestino) {
                        $nrtipotransfer = $nrtipotransferdestino; //A estrutura de destino é de outro tipo
                    }
                    // Verifica se há um ou mais registros com a mesma data de inicio e tipo de estrutura
                    try {
                        $this->validaExisteDataMovimentacao($nrorg, $nrvinculom, $dtinimoviment, $dtfimmoviment, $nrtipoestrutura, null, $nrtipotransfer);
                    } catch (\Exception $e) {
                        $msg_vinculos .= " - $nrvinculom <br>";
                        continue;
                    }

                    // Atualiza a data final da movimentação de origem para o dia anterior à nova movimentação
                    if ($movimentacao->getNrtpmovtransfm() != 9) { // Exceto transferencia de saida onde é utilizada mesma data de início e fim
                        $fimmoviment = DateUtil::subtraiIntervalo($dtinimovimentacao, 1);
                        $movimentacao->setDtfimmoviment($fimmoviment);
                        $this->entityManager->persist($movimentacao);
                    }

                    if ($dtfimmoviment == '') {
                        $dtfimmovimentacao = null;
                    } else {
                        $dtfimmovimentacao = DateUtil::getDataDeString($dtfimmoviment, DateUtil::FORMATO_BRASILEIRO, true);
                    }

                    //insere nova movimentação para os vinculos
                    $this->incluiMovimentacao($nrorg, $nrtipotransfer, $nrvinculom, $nrestruturam, $dtinimovimentacao,
                        $nrtipoestrutura, $nrtpmovtransfm, $nrmotivotransf, null, $idatualizaacerto, $dtfimmovimentacao, $dsobservacao);
                        
                    
                    //insere novo histórico caso na exista para a competencia
                    $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom);
                    //atualiza histórico do vínculo
                    $this->atualizaEstruturaHistoricoVinculo($nrorg, $nrvinculom, $nrtipoestrutura, $nrestruturam, $nrtipotransfer);

                    $this->entityManager->flush();
                    $inclusao = true;
                }
            }
            if($msg_vinculos != ""){
                $response->addMessage(new Response\Message(Messages::evaluateMessage(Messages::HCM00000_GER_MOVIMENTACAO_DATA_ALREADY_EXIST_LIST, $msg_vinculos), Response\Message::TYPE_ERROR));
            }
            if($inclusao){
                $response->addNotification(new Response\Notification(Messages::HCM00000_GER_MOVIMENTACAO_GERACAO_SUCCESS, Response\Notification::TYPE_SUCCESS));
            }else{
                $response->addNotification(new Response\Notification(Messages::HCM00000_GER_MOVIMENTACAO_GERACAO_ALERT, Response\Notification::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function setDataInicialUltimaMovimentacaoLegalParaCadaVinculo($movimentacoes): void
    {
        foreach ($movimentacoes as $movimentacao) {
            $this->dataInicialUltimaMovimentacaoLegalVinculo[$movimentacao->getNrvinculom()] = $movimentacao->getDtinimoviment();

            $this->setDataInicialUltimaMovimentacaoLegalVinculo($movimentacao);
        }
    }

    private function setDataInicialUltimaMovimentacaoLegalVinculo($movimentacao): void
    {
        $dataCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true)->format('Ymd');

        foreach ($this->dataInicialUltimaMovimentacaoLegalVinculo as $nrVinculom => $maiorDataInicial) {
            $isUltimaMovimentacaoLegal = (
                ($movimentacao->getDtinimoviment()->format('Ymd') > $maiorDataInicial) &&
                ($movimentacao->getDtinimoviment()->format('Ymd') <= $dataCompetencia) &&
                $nrVinculom == $movimentacao->getNrvinculom()
            );

            if ($isUltimaMovimentacaoLegal) {
                $this->dataInicialUltimaMovimentacaoLegalVinculo[$movimentacao->getNrvinculom()] = $movimentacao->getDtinimoviment();
            }
        }
    }

    private function setMovimentacaoSaida($movimentacao, $params): void
    {
        $nrtipotransfer = $params['NRTIPOTRANSFER'];
        $nrmotivotransf = $params['NRMOTIVOTRANSF'];
        $dtinimoviment = $params['DTINIMOVIMENT'];
        $idatualizaacerto = $params['IDATUALIZAACERTO'];
        $nrtipoestrutura = $params['NRTIPOESTRUTURA'];
        $dsobservacao = $params['DSOBSERVACAO'];
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrvinculom = $movimentacao->getNrvinculom();
        $dtinimovimentacao = DateUtil::getDataDeString($dtinimoviment, DateUtil::FORMATO_BRASILEIRO, true);
        $fimmoviment = DateUtil::subtraiIntervalo($dtinimovimentacao, 1);

        $this->incluiMovimentacao(
            $nrorg,
            $nrtipotransfer,
            $nrvinculom,
            $movimentacao->getNrestruturam(),
            $dtinimovimentacao,
            $nrtipoestrutura,
            9,
            $nrmotivotransf,
            null,
            $idatualizaacerto,
            $fimmoviment,
            $dsobservacao
        );
    }

    /**
     * Inclui movimentacao do vinculo.
     *
     * @param integer    $nrOrg            Numero da Organizacao.
     * @param integer    $nrTipoTransfer   Numero do Tipo de transferencia.
     * @param integer    $nrVinculom       Numero do Vinculo.
     * @param integer    $nrEstruturam     Numero da Estrutra.
     * @param string     $dtIniMoviment    Data inicial da movimentacao do vinculo.
     * @param integer    $nrTipoEstrutura  Numero do Tipo de Estrutura.
     * @param integer    $nrTpMovTransf    Numero do Tipo de Movimentacao de Transferencia.
     * @param DateTime   $dtRetroativa     Data Retroativa.
     * @param string     $idAtualizaAcerto Identificar de atualizacao de acerto (normal = null, excluir = 1 ou incluir = 2).
     * @param string     $dtFimMovimento   Data final da movimentacao do vinculo.
     *
     * @return $this->gpeMovimentacao.
     */
    public function incluiMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $nrTpMovTransf = null, $nrMotivoTransf = null, $dtRetroativa = null, $idAtualizaAcerto = 0,
        $dtFimMovimento = null, $dsObservacao = null
    ){
        
        $movimentacao = new $this->gpeMovimentacao;
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setNrtpmovtransfm($nrTpMovTransf);
        $movimentacao->setNrmotivotransf($nrMotivoTransf);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizaAcerto);
        $movimentacao->setDtfimmoviment($dtFimMovimento);
        $movimentacao->setDsobservacao($dsObservacao);

        $movimentacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtinclusao(DateUtil::getDataAtual());
        $movimentacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $movimentacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtultatu(DateUtil::getDataAtual());
        $movimentacao->setCdoperultatu($this->hcmParameters->getCdoperador());

        $movimentacao->setNrmovimentacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true));

        $this->entityManager->persist($movimentacao);
        $this->entityManager->flush();

        // Retorna movimentacao
        return $movimentacao;
    }
    
    public function manipulaMovimentacao(
        $nrorg, $nrVinculom, $nrEstruturaLegal, $nrEstruturaGerencial, $dtInicioMovimentacao, $mesCompetencia, $acao, $nrtpdemissao = null
    ){
        try{
            $this->entityManager->beginTransaction();
            $nrTpmovtransfm = $this->setTipoRecisao($nrtpdemissao);
            $dtInicioMovimentacao = Util::datadeString($dtInicioMovimentacao,true);
            $mesCompetencia = Util::datadeString($mesCompetencia,true);
            
            if ($nrEstruturaLegal == null) {
                $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrorg,
                   'nrvinculom' => $nrVinculom));
                
                $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrVinculom);
                $nrEstruturaLegal = $gpevinculoh->getNrestrutlegal();
            }

            $gpemovimentacao = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->findOneBy(array('nrorg' => $nrorg,
                   'nrvinculom' => $nrVinculom,'nrtpmovtransfm' => array(3,4,5,6,7,11)),(array('dtinimoviment' => 'DESC')));
            
            if (is_object($gpemovimentacao)) {
                if (!$dtInicioMovimentacao) {
                    $this->entityManager->remove($gpemovimentacao);
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                    $this->valeTransporteServices->setReservaValeTransporte($nrorg, $nrVinculom, 'N');
                    return;
                } else if ($dtInicioMovimentacao != $gpemovimentacao->getDtinimoviment() || $nrTpmovtransfm != $gpemovimentacao->getNrtpmovtransfm() ) {
                    $gpemovimentacao->setDtinimoviment($dtInicioMovimentacao);
                    $gpemovimentacao->setNrtpmovtransfm($nrTpmovtransfm);
                    $this->valeTransporteServices->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
                    $this->entityManager->persist($gpemovimentacao);
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                    return;
                }
            } 
            if($acao !== self::ALTERA_VINCULO && $dtInicioMovimentacao) {
                if ($nrEstruturaLegal) {
                    $movimentacaoLegal = $this->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFLEGAL, $nrVinculom, $nrEstruturaLegal, $dtInicioMovimentacao, $this->hcmParameters->getTipoEstruturaLegalOrg(), $nrTpmovtransfm);
                }
                if ($nrEstruturaGerencial) {
                    $movimentacaoGeren = $this->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFGEREN, $nrVinculom, $nrEstruturaGerencial, $dtInicioMovimentacao, $this->hcmParameters->getTipoEstruturaGerencialOrg(), $nrTpmovtransfm);
                }
                $this->valeTransporteServices->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
            }
            else if($dtInicioMovimentacao && $acao === self::ALTERA_VINCULO) {
                if (DateUtil::comparaDatas($dtInicioMovimentacao,$mesCompetencia)) {
                    // alteracao
                    if ($nrEstruturaLegal) {
                        $this->alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstruturaLegal, $dtInicioMovimentacao, GpeTipotransfer::TIPOTRANSFLEGAL);
                    }

                    if ($nrEstruturaGerencial) {
                        $this->alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstruturaGerencial, $dtInicioMovimentacao, GpeTipotransfer::TIPOTRANSFGEREN);
                    }
                    $this->valeTransporteServices->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
                }
            }
            unset($dtAux);
            unset($vinculoLogic);
            $this->entityManager->flush();
            $this->entityManager->commit();

        }catch (Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function setTipoRecisao($nrtpdemissao){
        switch($nrtpdemissao)
        {
            case 1;
                return 4;
            case 2;
                return 3;
            case 3;
                return 11;
            case 4;
                return 4;
            case 5;
                return 5;
            case 6;
                return 12;
            case 7;
                return 12;
            case 10;
                return 7;
            case 11;
                return 7;
            case 12;
                return 7;
            case 13;
                return 6;
            case 14;
                return 6;
            case 15;
                return 6;
            case 16;
                return 6;
            case 17;
                return 6;
            case 18;
                return 6;
            case 19;
                return 6;
            case 21;
                return 5;
            case 24;
                return 5;
            case 25;
                return 5;

            default;
                return 3;
        }
    }
    
    public function alteraDataIniMovimentacao($nrOrg, $nrVinculom, $antigaDtInicioMovimentacao, $novaDtInicioMovimeacao){
        try{
            $antigaDtInicioMovimentacao = Util::datadeString($antigaDtInicioMovimentacao,true);
            $novaDtInicioMovimeacao = Util::datadeString($novaDtInicioMovimeacao,true);
            
            $listaMovimentacoes = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(array('nrorg' => $nrOrg, 'nrvinculom' => $nrVinculom, 'dtinimoviment' => $antigaDtInicioMovimentacao));
            
            if($listaMovimentacoes){
                foreach($listaMovimentacoes as $altMovimentacao){
                    $altMovimentacao->setDtinimoviment($novaDtInicioMovimeacao);
                    $this->entityManager->persist($altMovimentacao);
                    $this->entityManager->flush();
                }
            }
            
        }catch (Exception $e){
            throw $e;
        }
    }
    
    public function alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstrutura, $dtInicioMovimentacao, $tpTransferencia){
        try{
            
            $objEstrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg,
                   'nrestruturam' => $nrEstrutura));
            $nrTipoEstrutura = $objEstrutura->getNrtipoestrutura();
            unset($objEstrutura);
            
            $listaMovimentacoes = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(array('nrorg' => $nrorg,
                   'nrvinculom' => $nrVinculom));
            foreach ($listaMovimentacoes as $altMovimentacao){
                if ($dtInicioMovimentacao === $altMovimentacao->getDtinimoviment()
                    && $altMovimentacao->getNrestruturam() !== $nrEstrutura
                    && $altMovimentacao->getNrtipotransfer() === (string)$tpTransferencia
                ){
                    $this->verificaTipoTransferencia($tpTransferencia, $nrTipoEstrutura);
                    $altMovimentacao->setNrestruturam($nrEstrutura);
                    $altMovimentacao->setNrtipoestrutura($nrTipoEstrutura);
                    $altMovimentacao->persist();
                }
            }
            unset($movimentacao);
            unset($paramorg);

        }catch (Exception $e){
            throw $e;
        }
    }
    
    public function getMovimentacoesCaged($nrorg, $nrtipoestrutura, $nrvinculom, $nrestrutlegal = null) {
        
        return  $this->movimentacaoRepository->getMovimentacoesCaged($nrorg, $nrtipoestrutura, $nrvinculom, $nrestrutlegal);
        
    }
    
    public function getMovimentacao($nrorg, $nrtipoestrutura, $nrvinculom, $nrtipotransfer) {
        return $this->movimentacaoRepository->getMovimentacao($nrorg, $nrtipoestrutura, $nrvinculom, $nrtipotransfer);
    }
    
    public function baixarPlanilhaMovimentacao() {
        $cabecalho = array(
            "Nr. Vínculo",
            "Matrícula",
            "Matrícula eSocial",
            "Nr. Tipo Transferência",
            "Nr. Transferência",
            "Nr. Estrutura",
            "Observação",
            "Dt. Fim Movimentação",
            "Dt. Início Movimentação",
            "Nr. Tipo Estrutura",
            "Dt. Mov. Retroativa",
            "Nr. Motivo da Transferência"
        );
        
        return $this->exportacaoExcelService->gravarTemplate_1($this->hcmParameters->getNrorgTrabalho(), $cabecalho, 'Movimentacao');
    }
    
    public function importarPlanilhaMovimentacao($row) {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        $dados       = $this->getDadosMovimentacao($row['DSPATH'][0]);
        foreach ($dados as $lin => $dado) {
            if (count($this->errosPlanilha($dado, $lin + 2)) > 0) {
                foreach ($this->errosPlanilha($dado, $lin + 2) as $msg) {
                    $messages[] = $msg;
                }
                continue;
            }
            if (count(explode('/', $dado['Dt. Mov. Retroativa'])) !== 3) {
                $dado['Dt. Mov. Retroativa'] = null;
            }
            if (count(explode('/', $dado['Dt. Fim Movimentação'])) !== 3) {
                $dado['Dt. Mov. Retroativa'] = null;
            }
            $nrestruturalegal = $this->retornaMovimentacaoLegalEntrada($nrorg, $this->hcmParameters->getTipoEstruturaLegalOrg(), $dado['Nr. Vínculo']);
                
            if($nrestruturalegal)
            {
                $valida = $this->validaHierarquiaGerenLegal($nrorg, $nrestruturalegal, $dado['Nr. Estrutura'], $dado['Nr. Tipo Transferência'], $dado['Nr. Tipo Estrutura'], $this->hcmParameters->getTipoEstruturaGerencialOrg(), $this->hcmParameters->getTipoEstruturaLegalOrg());
                if($valida == false)
                {
                    $messages[] = array("msg" => "Não há Hierarquia relacionada a Estrutura Gerencial escolhida com a Estrutura Legal atual do vínculo!", "row" => $lin + 2);
                    continue;
                }
            }
            if ($this->existeMesmaMovimentacao($dado)) {
                $messages[] = array("msg" => "Já existe essa movimentação para esse vínculo!", "row" => $lin + 2);
                continue;
            } else {
                
                $res = $this->incluiMovimentacao(
                    $nrorg,
                    $dado['Nr. Tipo Transferência'],
                    $this->getNumeroVinculo($dado),
                    $dado['Nr. Estrutura'],
                    DateUtil::getDataDeString($dado['Dt. Início Movimentação'], DateUtil::FORMATO_BRASILEIRO, true),
                    $dado['Nr. Tipo Estrutura'],
                    $dado['Nr. Transferência'],
                    $dado['Nr. Motivo da Transferência'] == 0 ? null : $dado['Nr. Motivo da Transferência'],
                    DateUtil::getDataDeString($dado['Dt. Mov. Retroativa'], DateUtil::FORMATO_BRASILEIRO, true),
                    0,
                    DateUtil::getDataDeString($dado['Dt. Fim Movimentação'], DateUtil::FORMATO_BRASILEIRO, true),
                    substr($dado['Observação'], 0, 2000)
                );
                $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $dado['Nr. Vínculo']);
                $this->atualizaEstruturaHistoricoVinculo($nrorg, $dado['Nr. Vínculo'], $dado['Nr. Tipo Estrutura'], $dado['Nr. Estrutura'], $dado['Nr. Tipo Transferência']);
            }
        }
        
        return $messages;
    }
    
    public function errosPlanilha($dado, $linha) {
        $mensagens = [];
        
        if ($this->getNumeroVinculo($dado) == null) {
            $mensagens[] = array("msg" => "Nenhum vínculo foi encontrado", "row" => $linha);
        };
        
        if ($this->getEstrutura($dado) == null) {
            $mensagens[] = array("msg" => "Nenhuma estrutura foi encontrada", "row" => $linha);
        }
        
        if ($this->getTipoTransferencia($dado) == null) {
            $mensagens[] = array("msg" => "Tipo de transferência inválido", "row" => $linha);
        }
        
        if ($this->getMotivoTransferencia($dado) == null) {
            $mensagens[] = array("msg" => "Nr. Transferência inválido", "row" => $linha);
        }

        if ($this->getDataInicioMovimentacao($dado) == null) {
            $mensagens[] = array("msg" => "Dt. Início movimentação inválida", "row" => $linha);
        }
        
        return $mensagens;
    }
    
    public function getDadosMovimentacao($path) {
        $head = array(
                'Nr. Vínculo',
                'Matrícula',
                'Matrícula eSocial',
                'Nr. Tipo Transferência',
                'Nr. Transferência',
                'Nr. Estrutura',
                'Observação',
                'Dt. Fim Movimentação',
                'Dt. Início Movimentação',
                'Nr. Tipo Estrutura',
                'Dt. Mov. Retroativa',
                'Nr. Motivo da Transferência'
            );
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        $dateColumns = array('Dt. Fim Movimentação', 'Dt. Início Movimentação', 'Dt. Mov. Retroativa');
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('Nr. Vínculo', 'Matrícula', 'Matrícula eSocial'), $dateColumns);
    }
    
    public function existeMesmaMovimentacao ($dado) {
        $movimentacoes = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(
            array(
                'nrvinculom' => $dado['Nr. Vínculo'], 
                'nrorg' => $this->hcmParameters->getNrorgTrabalho(),
                'nrtipotransfer' => $dado['Nr. Tipo Transferência'], 
                'nrtipoestrutura' => $dado['Nr. Tipo Estrutura']
            )
        );
        
        foreach ($movimentacoes as $movimentacao) {
            
            $existe = (
                $movimentacao->getDtinimoviment()->format('d/m/Y') == $dado['Dt. Início Movimentação'] &&
                $movimentacao->getNrtipoestrutura() == $dado['Nr. Tipo Estrutura'] &&
                $movimentacao->getNrestruturam() == $dado['Nr. Estrutura']
            );
            
            if ($existe == true) {
                return true;
            }
        }
        
        return false;
    }
    
    public function getNumeroVinculo($dado) {
        if ($dado['Nr. Vínculo']) {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                array(
                    'nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 
                    'nrvinculom'=>$dado['Nr. Vínculo']
                )
            );
            
            if ($this->existeVinculo($vinculom)) {
                return $vinculom->getNrvinculom();
            } else {
                return null;
            }
        }
        
        if ($dado['Matrícula']) {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                array(
                    'nrorg'=>$this->hcmParameters->getNrorgTrabalho(), 
                    'cdmatricula'=>$dado['Matrícula']
                )
            );
            
            if ($this->existeVinculo($vinculom)) {
                return $vinculom->getNrvinculom();
            } else {
                return null;
            }
        }
        
        if ($dado['Matrícula eSocial']) {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial(
                $this->hcmParameters->getNrorgTrabalho(), 
                null, 
                $dado['Matrícula eSocial'],
                DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
                null
            );
            
            if ($this->existeVinculo($vinculom)) {
                return $vinculom->getNrvinculom();
            } else {
                return null;
            }
        }
    }
    
    function existeVinculo($obj) {
        if ($obj == null) {
            return false;
        } else {
            return true;
        }
    }
    
    public function getEstrutura($dado) {
        return $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
            array(
                'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 
                'nrestruturam' => $dado['Nr. Estrutura'],
                'nrtipoestrutura' => $dado['Nr. Tipo Estrutura']
            )
        );
    }
    
    public function getTipoTransferencia($dado) {
        $tipoTransferencia = array("1" => "Movimentação Legal", "2" => "Movimentação Gerencial");
        
        if (array_key_exists((int)$dado['Nr. Tipo Transferência'], $tipoTransferencia)) {
            return $tipoTransferencia[$dado['Nr. Tipo Transferência']];
        } else {
            return null;
        }
    }
    
    public function getMotivoTransferencia($dado) {
        return $this->entityManager->getRepository(Repositories::GPE_TPMOVTRANSFM)->findOneBy(
            array(
                "nrtpmovtransfm" => $dado['Nr. Transferência'], 
                "nrorg" => array(
                    $this->hcmParameters->getNrorgTrabalho(),
                    $this->hcmParameters->getNrorgPadrao()
                )
            )
        );
    }
    
    public function getDataInicioMovimentacao($dado) {
        if ($dado['Dt. Início Movimentação'] == null) {
            return null;
        }
        
        if (count(explode('/',$dado['Dt. Início Movimentação'])) !== 3) {
            return null;
        }
        
        return $dado['Dt. Início Movimentação'];
    }
} 